from crontab import CronTab
from OnlySnarf.settings import SETTINGS as settings

################
##### Cron #####
################

def delete(comment):
    cron = CronTab(user=str(settings.CRON_USER))
    cron.remove_all(comment=comment)
    cron.write()
    print("Cron Deleted: {}".format(comment))

def deleteAll():
    cron = CronTab(user=str(settings.CRON_USER))
    cron.remove_all()
    cron.write()
    print("Crons Deleted")

# use cron.comment to set cron name and find crons
def disable(comment):
    # find cron by comment
    cron = find(comment)
    cron.enable(False)
    cron.write()
    print("Cron Disabled: {}".format(comment))

def enable(comment):
    # find cron by comment
    cron = find(comment)
    cron.enable()
    cron.write()
    print("Cron Enabled: {}".format(comment))

def create(comment, args=[], minute=None, hour=None):
    print("Creating Cron: {}".format(comment))
    # if find(comment) is not None:
        # print("Warning: Cron Exists")
    cron = CronTab(user=str(settings.CRON_USER))
    cron.remove_all(comment=comment)
    args = [n.strip() for n in args]
    newCron = cron.new(command="/usr/local/bin/onlysnarfpy -cron {} {} >> /var/log/onlysnarf.log 2>&1".format(comment, " ".join(args)), comment=comment);
    newCron.hour.every(1)
    if minute is not None:
        newCron.minute.on(minute)
    if hour is not None:
        newCron.hour.on(hour)
    cron.write()
    print("Created Cron: {}".format(comment))

def list():
    cron = CronTab(user=str(settings.CRON_USER))
    print("Crons:")
    for job in cron:
        print(job)

def find(comment):
    cron = CronTab(user=str(settings.CRON_USER))
    iter1 = cron.find_comment(str(comment))
    for item in iter1:
        return item
    return False 

def getAll():
    cron = CronTab(user=str(settings.CRON_USER))
    jobs = []
    for job in cron:
        jobs.append(job)
    return jobs

###################
##### Special #####
###################

def discount(user, amount=None, months=None):
    args = ["-action","discount","-user",str(user)]
    if amount: 
        args.append(["-amount"])
        args.append([amount])
    if months:
        args.append(["-months"])
        args.append([months])
    create("discount-"+user, args)

def message(user, text=None, image=None, price=None):
    args = ["-action","message","-user",str(user)]
    if text: 
        args.append(["-text"])
        args.append(["\"{}\"".format(text)])
    if image:
        args.append(["-image"])
        args.append(["\"{}\"".format(image)])
    if price:
        args.append(["-price"])
        args.append(["\"{}\"".format(price)])
    create("message-"+user, args)

def post(text):
    args = ["-action","post","-text","\"{}\"".format(text)]
    create("post-"+user, args)

def upload(opt):
    args = ["-type",str(opt)]
    create("upload", args)

def uploadInput(type_, path):
    args = ["-type",str(type_),"-method","input","-input","\"{}\"".format(path)]
    create("upload-input"+str(path), args)

# check all scenes folders' data.txt's for "releaseDate"
def checkScenes():
    pass

# sends messages to users that were queued for a specific time
def sendQueuedMessages():
    pass

###############
##### Dev #####
###############

def test():
    create("upload-video", args=["-type","video"])
    # create("upload-balls", [], "30", "11")
    # create("check-scenes")
    # disable("upload-video")
    # enable("upload-video")
    return True


# Cron:

# Message:
# - All, etc

# Upload:
# - Gallery
# - Image
# - Video
# - Performer
# - Scene


#!/usr/bin/python3
# 3/28/2019 Skeetzo

import random
import os
import shutil
import datetime
import json
import sys
import pathlib
import time
from OnlySnarf.settings import SETTINGS as settings
from OnlySnarf import google as Google
from OnlySnarf import driver as OnlySnarf
from OnlySnarf.user import User
from OnlySnarf import cron as Cron

#################################################################
#################################################################
#################################################################

FIFTY_MEGABYTES = 50000000

####################
##### Discount #####
####################

def discount(choice, depth=1, amount=None, months=None):
    if not amount: amount = input("Discount: ")
    if not months: months = input("Months: ")
    users = []
    if str(choice) == "all":
        users = User.get_all_users()
    elif str(choice) == "new":
        users = User.get_new_users()
    elif str(choice) == "favorite":
        users = User.get_favorite_users()
    elif str(choice) == "recent":
        users = User.get_recent_users()
    else:
        if isinstance(choice, str):
            user = User.get_user_by_username(choice)
        users.append(user)
    for user in users:
        # try:
        success = OnlySnarf.discount_user(user.id, depth=depth, discount=amount, months=months)
        if not success: print("Error: There was an error discounting - {}/{}".format(user.id, user.username))
        # except Exception as e:
            # settings.maybePrint(e)
        depth = int(depth) + 1
    OnlySnarf.exit()
    return True

####################
##### Download #####
####################

def download(fileChoice, methodChoice="random", file=None, folderName=None, parent=None):
    if methodChoice == "random":
        return Google.random_download(fileChoice)
    elif methodChoice == "choose" and file is not None:
        if fileChoice == 'image' or fileChoice == 'video':
            return Google.download_file(file)
        elif fileChoice == 'gallery':
            return Google.download_gallery(file)
        elif fileChoice == 'performer':
            if "folder" in file.get("mimeType"):
                return Google.download_content(file)
            else:          
                return Google.download_file(file)  
#################################################################
        elif fileChoice == 'scene':
            return Google.download_scene(file)
#################################################################
    else:
        print("Error: Unable to Download")
        return None

###################
##### Message #####
###################

def message(choice, message=None, image=None, price=None, username=None):
    if str(choice) == "all":
        print("Messaging: All")
        users = User.get_all_users()
    elif str(choice) == "recent":
        print("Messaging: Recent")
        users = User.get_recent_users()
    elif str(choice) == "favorites":
        print("Messaging: Recent")
        users = User.get_favorite_users()
    elif str(choice) == "new":
        print("Messaging: New")
        users = User.get_new_users()
    elif str(choice) == "user":
        print("Messaging: User - {}".format(username))
        if username is None:
            print("Error: Missing Username")
            return
        users = [User.get_user_by_username(str(username))]
    else:
        print("Error: Missing Message Choice")
        return
    if image == None and str(settings.METHOD) == "random": 
        images = Google.get_images()
        image = random.choice(images)
        image = Google.download_file(image[1]).get("path")
    success = False
    backup = False
    for user in users:
        if user:
            try:
                success = user.sendMessage(message=message, image=image, price=price)
                if not success: print("Error: There was an error messaging - {}/{}".format(user.id, user.username))
                if success: backup = True
            except Exception as e:
                settings.maybePrint(e)
    if backup:
        Google.upload_input(image)
    OnlySnarf.exit()
    return success
                
################
##### Post #####
################

def post(text=None, override=False):
    expires = settings.EXPIRES or ""
    schedule = settings.getSchedule()
    poll = None
    duration = settings.DURATION or ""
    questions = settings.QUESTIONS or []
    if not text: text = input("Text: ".format(text))
    else: print("Text: "+text)
    if not override:
        print("[Enter] or Text or Cancel")
        confirm = input()
        if confirm != "":
            if str(confirm) == "None" or str(confirm) == "Cancel" or str(confirm) == "C" or str(confirm) == "c":
                print("Canceling Post")
                return False
            else:
                text = confirm
        print("Expiration [1, 3, 7, 99 or 'No limit']:")
        expires_ = input("({})>> ".format(expires))
        if str(expires_) != "":
            expires = expires_
        schedule_ = input("Schedule (y/n): ")
        if str(schedule_) != "" and str(schedule_).lower() != "n":
            schedule_ = input( "({})>> ".format(schedule))
            date_ = settings.DATE or ""
            print("Date [mm/dd/YY]: ")
            date = input("({})>>".format(date_))
            time_ = settings.TIME or ""
            print("Time [HH:MM]: ")
            time = input("({})>>".format(time_))
            schedule = "{}:{}".format(date, time)
        questions_ = input("Poll (y/n): ")
        if str(questions_) != "" and str(questions_).lower() != "n":
            print("Duration [1, 3, 7, 99 or 'No limit']:")
            duration_ = input("({})>> ".format(duration))
            if str(duration_) != "":
                duration = duration_
            print("Questions:\n> {}".format("\n> ".join(questions)))
            questions_ = input(">> ")
            if str(questions_) != "":
                questions = questions_
        poll = {"period":duration,"questions":questions}
    try:
        successful = OnlySnarf.post(text, expires=expires, schedule=schedule, poll=poll)
        # if successful: print("Post Successful")
        # else: print("Post Failed")
        OnlySnarf.exit()
        return successful
    except Exception as e:
        settings.maybePrint(e)
    
#####################
##### Promotion #####
#####################

def give_trial(user):
    print("Applying Promotion: "+user)
    link = OnlySnarf.get_new_trial_link()
    text = "Here's your free trial link!\n"+link
    settings.maybePrint("Link: "+str(text))
    send_email(email, text)

def send_email(email, text):
    print("Sending Email: "+str(email))
    pass

#################
##### Reset #####
#################

# Deletes local file
def remove_local():
    try:
        if str(settings.SKIP_DELETE) == "True":
            print("Skipping Local Remove")
            return
        # print('Deleting Local File(s)')
        # delete /tmp
        tmp = settings.getTmp()
        if os.path.exists(tmp):
            shutil.rmtree(tmp)
            print('Local File(s) Removed')
        else:
            print('Local Files Not Found')
    except Exception as e:
        settings.maybePrint(e)

###################
##### Release #####
###################

def release(opt, methodChoice="random", file=None, folderName=None, parent=None):
    try:
        if str(methodChoice) != "input":
            print("0/3 : Deleting Locals")
            remove_local()
        sys.stdout.flush()
        print("1/3 : Running - {}".format(opt))
        released = release_(opt, methodChoice=methodChoice, file=file, folderName=folderName, parent=parent)
        # if released == False: print("Upload Failed")
        # else: print("Upload Successful")
        sys.stdout.flush()
        print('2/3 : Cleaning Up Files')
        remove_local()
        print('Files Cleaned ')
        print('3/3 : Google Drive to OnlyFans Upload Complete')
        OnlySnarf.exit()
        sys.stdout.flush()
        return released
    except Exception as e:
        settings.maybePrint(e)

def release_(opt, methodChoice="random", file=None, folderName=None, parent=None):
    try:
        if not opt:
            print("Error: Missing Option")
            return False
        print("Uploading: {}".format(opt))
        data = None
        if str(methodChoice) == "input":
            input_ = settings.getInput()
            if not input_: return False
            global FIFTY_MEGABYTES
            if int(os.stat(str(input_)).st_size) >= FIFTY_MEGABYTES or settings.FORCE_REDUCTION: # greater than 1GB
                input_ = Google.reduce(input_)
            data = {"path":str(input_),"text":str(settings.TEXT)}
        else:
            data = download(opt, methodChoice=methodChoice, file=file)
        if data == None:
            print("Error: Missing Data")
            return False
        text = None
        path = None
        keywords = []
        performers = []
        files = None
        expires = settings.EXPIRES or None
        schedule = settings.getSchedule()
        poll = None
        duration = settings.DURATION or ""
        questions = settings.QUESTIONS or []
        if parent: parent = parent.get("title")
        try:
            if file == None: file = data.get("file") or {}
            text = file.get("title") or data.get("text")
            path = data.get("path")
            files = data.get("files")
            keywords = data.get("keywords") or parent or settings.KEYWORDS
            performers = data.get("performers") or settings.PERFORMERS
            # if parent: keywords = parent.split(" ")
            if isinstance(keywords, str): keywords = keywords.split(" ")
            if str(opt) == "performer":
                keywords = folderName or keywords
                if parent: performers = parent.split(" ")
            if isinstance(keywords, list): keywords = [n.strip() for n in keywords]
            if str(methodChoice) == "choose":
                print("Text: ")
                text_ = input("({})>> ".format(text))
                if text_ != "":
                    text = text_
                print("Keywords: ")
                keywords_ = input("({})>> ".format(keywords))
                if keywords_ != "":
                    if str(keywords_) == "None":
                        keywords = []
                    elif str(keywords_) == "[]":
                        keywords = []
                    elif str(keywords_) == " ":
                        keywords = []
                    else:
                        keywords = keywords_.split(",")
                        keywords = [n.strip() for n in keywords]
                print("Performers: ")
                performers_ = input("({})>> ".format(performers))
                if performers_ != "":
                    if str(performers_) == "None":
                        performers = []
                    elif str(performers_) == "[]":
                        performers = []
                    elif str(performers_) == " ":
                        performers = []
                    else:
                        performers = performers_.split(",")
                        performers = [n.strip() for n in performers]
                print("Expiration [1, 3, 7, 99 or 'No limit']: ")
                expires_ = input("({})>> ".format(expires))
                if str(expires_) != "":
                    expires = expires_
                schedule_ = input("Schedule (y/n): ".format(schedule))
                if str(schedule_) != "" and str(schedule_) != "n":
                    date_ = settings.DATE or ""
                    print("Date [mm/dd/YY]: ")
                    date = input("({})>>".format(date_))
                    time_ = settings.TIME or ""
                    print("Time [HH:MM]: ")
                    time = input("({})>>".format(time_))
                    schedule = "{}:{}".format(date, time)
                questions_ = input("Poll (y/n): ")
                if str(questions_) != "" and str(questions_).lower() != "n":
                    print("Duration [1, 3, 7, 99 or 'No limit']:")
                    duration_ = input("({})>> ".format(duration))
                    if str(duration_) != "":
                        duration = duration_
                    print("Questions:\n> {}".format("\n> ".join(questions)))
                    questions_ = input(">> ")
                    if str(questions_) != "":
                        questions = questions_
                if len(questions) > 0:
                    poll = {"period":duration,"questions":questions}
                else: poll = None
        except Exception as e:
            settings.maybePrint(e)
        if path == None: print("Warning: Missing Content")
        if str(settings.DEBUG) == "True" and str(settings.VERBOSE) == "True":
            print("Data:")
            print("- Text: {}".format(text)) # name of scene
            print("- Keywords: {}".format(keywords)) # text sent in messages
            print("- Content: {}".format(path)) # the file(s) to upload
            print("- Performer(s): {}".format(performers)) # name of performers
            print("- Expiration: {}".format(expires)) # name of performers
            print("- Schedule: {}".format(schedule)) # name of performers
            print("- Poll: {}".format(poll)) # name of performers
        successful_upload = upload(path, text=text, keywords=keywords, performers=performers, expires=expires, schedule=schedule, poll=poll)
        if not successful_upload:
            pass
        elif files:
            Google.move_files(text, files)
        elif str(methodChoice) != "input":
            Google.move_file(file)
        elif str(methodChoice) == "input":
            Google.upload_input()
        return successful_upload
    except Exception as e:
        settings.maybePrint(e)
        return False
    
# upload a file or gallery
# send a message to [recent, all, user] w/ a preview image
def release_scene(methodChoice="random", file=None, folderName=None, parent=None):
    try:
        print("Releasing Scene")
        response = download("scene", methodChoice=methodChoice, file=file)
        if response == None:
            print("Error: Failure Releasing Scene")
            return False
        # settings.maybePrint("Scene: {}".format(response))
        content = response[0]
        preview = response[1]
        data = response[2]
        google_folder = response[3]
        # print("Data:\n{}".format(json.dumps(data, sort_keys=True, indent=4)))
        data = json.loads(json.dumps(data))
        settings.maybePrint("Data: {}".format(data))
        title = None
        message = None
        price = None
        text = None
        performers = None
        keywords = None
        users = None
        title = data["title"]
        message = data["message"]
        price = data["price"]
        text = data["text"]
        performers = data["performers"]
        keywords = data["keywords"]
        if str(keywords) == " " or str(keywords[0]) == " ":
            keywords = []
        users = data["users"]
        if title == None:
            print("Error: Missing Scene Title")
            return False
        if message == None:
            print("Error: Missing Scene Message")
            return False
        if price == None:
            print("Error: Missing Scene Price")
            return False
        if text == None:
            print("Error: Missing Scene Text")
            return False
        print("Scene:")
        print("- Title: {}".format(title)) # name of scene
        print("- Text: {}".format(text)) # text entered into file upload
        print("- Price: {}".format(price)) # price of messages sent
        print("- Message: {}".format(message)) # text sent in messages
        print("- Keywords: {}".format(keywords)) # text sent in messages
        print("- Performers: {}".format(performers)) # text sent in messages
        print("- Preview: {}".format(preview)) # image sent in messages
        print("- Content: {}".format(content)) # the file(s) to upload
        print("- Users: {}".format(users)) # the file(s) to upload 
        files = os.listdir(content)
        file = files[0]
        ext = str(os.path.splitext(file)[1].lower())
        settings.maybePrint('ext: '+str(ext))
        successful_upload = upload(path, text, keywords, performers)
        if successful_upload:
            if str(users[0]) == "all" or str(users[0]) == str("recent") or str(users[0]) == str("favorites"):
                users = users[0]
            if not users or str(users).lower() == "none":
                print("Warning: Missing User Choice")
            elif str(users) == "all" or str(users) == "recent" or str(users) == "favorites":
                successful_message = OnlySnarf.message(choice=str(users), message=message, image=preview, price=price)
            else:
                for user in users:
                    successful_message = OnlySnarf.message(choice="user", message=message, image=preview, price=price, username=user)
            if successful_message:
                Google.move_file(google_folder)
            else:
                print("Error: Failure Messaging")
                return False
        else:
            print("Error: Failure Uploading")
            return False
        return True
    except Exception as e:
        settings.maybePrint(e)
        return False

#################
##### Reset #####
#################

def reset():
    OnlySnarf.reset()

##################
##### Upload #####
##################

def upload(path, text="", keywords=[], performers=[], expires=None, schedule=None, poll=None):
    # settings.maybePrint("Uploading: {}".format(path))
    try:
        if not schedule: schedule = settings.getSchedule()
        if not poll: poll = settings.getPoll()
        successful = OnlySnarf.upload_to_OnlyFans(path=path, text=text, keywords=keywords, performers=performers, expires=expires, schedule=schedule, poll=poll)
        # if successful: print("Upload Successful")
        # else: print("Upload Failed")
        return successful
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Unable to Upload")
        return False

#################
##### Users #####
#################

def get_users():
    settings.maybePrint("Getting Users")
    try:
        return User.get_all_users()
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Unable to get users");
        return []

###############
##### Dev #####
###############

def test(TYPE, methodChoice="random", file=None, folderName=None, parent=None):
    print('0/3 : Deleting Locals')
    remove_local()
    print('1/3 : Testing')

    # ### Promotion ###
    print('TESTING: Cron')
    response = Cron.test()
    if not response or response == None:
        print("Error: Failed to test crons")
    reset_ = reset()
    if not reset_:
        return print("Error: Failed to Reset")
    return

################################################################################################################################################

def main():
    try:
        # os.system('clear')
        settings.initialize()
        success = False
        if str(settings.ACTION) == "upload":
            success = release(settings.TYPE, methodChoice=settings.METHOD)
        elif str(settings.ACTION) == "post":
            success = post(text=settings.TEXT, override=True)
        elif str(settings.ACTION) == "message":
            METHOD_ = settings.METHOD
            settings.METHOD = "random"
            success = message(METHOD_, message=settings.TEXT, image=settings.IMAGE, price=settings.PRICE, username=settings.USER)
        elif str(settings.ACTION) == "discount":
            if str(settings.USER) == "" or str(settings.USER) == "None": settings.USER = "all"
            success = discount(settings.USER, amount=settings.AMOUNT, months=settings.MONTHS)
        else:
            print("Warning: Missing Method")
        if success and str(settings.CRON) != "False":
            Cron.delete(settings.CRON)
    except Exception as e:
        print(e)
        print("Shnarf!")
    finally:
        sys.exit(0)

if __name__ == "__main__":
    main()
else:
    try:
        settings.initialize()
    except Exception as e:
        print(e)
        print("Shnnarf?")


#!/usr/bin/python3
# Global Settings
import re
import sys
import os
import json
import configparser

class Settings:
    def __init__(self):
        pass

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, val):
        return setattr(self, key, val)

    def initialize(self):
        # print("Initializing Settings")
        try:
            if self.INITIALIZED:
                # print("Already Initialized, Skipping")
                return
        except:
            self.INITIALIZED = False
        self.ACTION = "upload"
        self.AMOUNT = 0
        self.MONTHS = 0
        self.PRICE = 0
        self.METHOD = "random"
        self.CRON = False
        self.KEYWORDS = []
        self.PERFORMERS = []
        self.USERS_FAVORITE = []
        self.EXPIRES = None
        self.SCHEDULE = None
        self.DATE = None
        self.TIME = None
        self.DURATION = None
        self.QUESTIONS = []
        self.BYKEYWORD = None
        # -backup
        # backup uploaded content to "posted" folder
        self.BACKUP = False
        # -cron-user
        # the user to run OnlySnarf as
        self.CRON_USER = "root"
        # -force-backup
        # force Google backup
        self.FORCE_BACKUP = False
        # -debug
        # debugging - skips uploading and deleting unless otherwise forced
        self.DEBUG = False
        # default message for debugging
        self.DEFAULT_MESSAGE = ":)"
        # default price for user messages
        self.DEFAULT_PRICE = "10.00"
        # default user greeting
        self.DEFAULT_GREETING = "hi! thanks for subscribing :3 do you have any preferences?"
        # -debug-delay
        # user message delay
        self.DEBUG_DELAY = False
        self.DEBUG_DELAY_AMOUNT = 10
        # -delete-google
        # delete uploaded content
        self.DELETE_GOOGLE = False
        # -create-drive 
        # creates missing OnlySnarf folders in Google Drive
        self.CREATE_DRIVE = False
        # configurable w/ profile.conf
        # OnlySnarf Drive folder list
        self.DRIVE_FOLDERS = [
            "images",
            "galleries",
            "performers",
            "scenes",
            "videos"
        ]
        # -force-delete
        # force Google file deletion upon upload
        self.FORCE_DELETE = False
        # -force-reduce
        # force mp4 reduction
        self.FORCE_REDUCTION = False
        # -force-upload
        # ignore upload max wait
        self.FORCE_UPLOAD = False
        # -image-path
        # path to local file to use
        self.PATH_LOCAL = None
        # -input
        # path to local file(s) to upload
        self.INPUT = None
        # -image
        # path to local image to use for message or upload
        self.IMAGE = None
        # -image-limit
        # maximum number of images to upload
        self.IMAGE_UPLOAD_LIMIT = 6
        # -image-max
        # maximum number of images that can be uploaded
        self.IMAGE_UPLOAD_MAX = 20
        # maximum number of images that can be uploaded in a message
        self.IMAGE_UPLOAD_MAX_MESSAGES = 5
        # -overwrite-local
        # self.OVERWRITE_LOCAL = False
        # -mount-path
        # the mounth path for a local directory of OnlyFans config files
        self.MOUNT_PATH = None
        # -password
        # the password for the OnlyFans / Twitter
        self.PASSWORD = None
        # -drive-path
        # the folder path within Google Drive for OnlySnarf's root folder
        self.DRIVE_PATH = None
        # -user-path
        # the path to the users.json file
        self.USERS_PATH = "users.json"
        # -config-path
        # the path to the config.json file
        self.PATH_CONFIG = "config.json"
        # -google-path
        # the path to the google_creds.txt
        self.GOOGLE_PATH = "google_creds.txt"
        # the path to the client_secret.json
        self.PATH_SECRET = "client_secret.json"
        # -prefer-local
        # prefers local cache over refreshing first call
        self.PREFER_LOCAL = False
        # the maximum number of recent users
        self.RECENT_USER_COUNT = 3
        # can be set in profile.conf
        # root Google drive folder
        self.ROOT_FOLDER = "OnlySnarf"
        # -save-users
        # saves OnlyFans users upon exit
        self.SAVE_USERS = False
        # -skip-backup
        # skips file backup if toggled by .conf
        self.SKIP_BACKUP = False
        # -skip-delete
        # skip local file deletion before and after upload
        self.SKIP_DELETE = False
        # -skip-download
        self.SKIP_DOWNLOAD = False
        # -skip-reduce
        # skip mp4 reducing
        self.SKIP_REDUCE = False
        # -skip-repair
        # skip mp4 repairs
        self.SKIP_REPAIR = False
        # can be set in profile.conf
        # -skip-upload
        # skips file upload
        self.SKIP_UPLOAD = False
        # list of users to skip
        self.SKIP_USERS = []
        # -show 
        # shows window
        self.SHOW_WINDOW = False
        # -text
        # text for message or upload
        self.TEXT = None
        # fixes thumbnail preview
        self.THUMBNAILING_PREVIEW = False
        # -type
        # the type of upload
        self.TYPE = None
        # -tweet
        # enabled tweeting
        self.TWEETING = False
        # -user
        # the user to target
        self.USER = None
        # user id found in OnlyFans
        self.USER_ID = None
        # -username
        # the OnlyFans / Twitter username to use
        self.USERNAME = None
        # -verbose
        # more output
        self.VERBOSE = False
        # custom repair option for shitty gopro videos
        self.WORKING_VIDEO = "video.mp4"
        # config file
        if os.path.exists("/etc/onlysnarf/config.conf"):
            readConf(self, "/etc/onlysnarf/config.conf")
        elif os.path.exists(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.conf")):
            readConf(self, os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.conf"))
        i = 0
        while i < len(sys.argv):
            sys.argv[i] = sys.argv[i][1:] # remove - in front
            truths_ = ["SKIP_BACKUP","BACKUP","CREATE_DRIVE","DEBUG","DEBUG_DELAY","DELETE_GOOGLE","FORCE_DELETE","FORCE_UPLOAD","FORCE_REDUCTION","PREFER_LOCAL","SAVE_USERS","SHOW_WINDOW","SKIP_DELETE","SKIP_DOWNLOAD","SKIP_REDUCE","SKIP_REPAIR","SKIP_UPLOAD","TWEETING","VERBOSE","THUMBNAILING_PREVIEW"]
            falses_ = []
            nexts_ = ["BYKEYWORD","PERFORMERS","KEYWORDS","DURATION","QUESTIONS","DATE","TIME","SCHEDULE","EXPIRES","USERS_FAVORITE","CRON","METHOD","PRICE","AMOUNT","MONTHS","ACTION","CRON_USER","INPUT","IMAGE","IMAGE_UPLOAD_LIMIT","IMAGE_UPLOAD_MAX","TYPE","TEXT","USER","DRIVE_PATH","GOOGLE_PATH","MOUNT_PATH","USERS_PATH","USERNAME","PASSWORD","USER_ID"]
            j = 0
            while j < len(truths_):
                if str(truths_[j]).upper() in str(sys.argv[i]).upper().replace("-","_"):
                    # self.set(truths_[j], True)
                    self[truths_[j]] = True
                j = j + 1
            j = 0
            while j < len(falses_):
                if str(falses_[j]).upper() in str(sys.argv[i]).upper().replace("-","_"):
                    # self.set(falses_[j], False)
                    self[falses_[j]] = False
                j = j + 1
            j = 0
            while j < len(nexts_):
                if str(nexts_[j]).upper() in str(sys.argv[i]).upper().replace("-","_"):
                    # self.set(nexts_[j], sys.argv[i+1])  
                    try:
                        self[nexts_[j]] = sys.argv[i+1]
                    except Exception as e:
                        # print(e)
                        pass
                j += 1
            i += 1
        if self.MOUNT_PATH is not None:
            self.PATH_CONFIG = os.path.join(self.MOUNT_PATH, self.PATH_CONFIG)
            self.PATH_SECRET = os.path.join(self.MOUNT_PATH, self.PATH_SECRET)
            self.GOOGLE_PATH = os.path.join(self.MOUNT_PATH, self.GOOGLE_PATH)
            self.USERS_PATH = os.path.join(self.MOUNT_PATH, self.USERS_PATH)
            self.WORKING_VIDEO = os.path.join(self.MOUNT_PATH, self.WORKING_VIDEO)
        else:
            self.PATH_CONFIG = os.path.join(os.path.dirname(os.path.realpath(__file__)), self.PATH_CONFIG)
            self.PATH_SECRET = os.path.join(os.path.dirname(os.path.realpath(__file__)), self.PATH_SECRET)
            self.GOOGLE_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), self.GOOGLE_PATH)
            self.USERS_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), self.USERS_PATH)
            self.WORKING_VIDEO = os.path.join(os.path.dirname(os.path.realpath(__file__)), self.WORKING_VIDEO)
        if self.KEYWORDS != "":
            if str(self.KEYWORDS) == "None":
                self.KEYWORDS = []
            elif str(self.KEYWORDS) == "[]":
                self.KEYWORDS = []
            elif str(self.KEYWORDS) == " ":
                self.KEYWORDS = []
            else:
                self.KEYWORDS = self.KEYWORDS.split(",")
                self.KEYWORDS = [n.strip() for n in self.KEYWORDS]
        if self.PERFORMERS != "":
            if str(self.PERFORMERS) == "None":
                self.PERFORMERS = []
            elif str(self.PERFORMERS) == "[]":
                self.PERFORMERS = []
            elif str(self.PERFORMERS) == " ":
                self.PERFORMERS = []
            else:
                self.PERFORMERS = self.PERFORMERS.split(",")
                self.PERFORMERS = [n.strip() for n in self.PERFORMERS]

        self.INITIALIZED = True
        # print("Settings Initialized")
    ###################################################

    #####################
    ##### Functions #####
    #####################

    def getInput(self):
        if str(self.INPUT) == "None":
            self.maybePrint("Error: Missing Local Input")
            return False
        if os.path.isdir(str(self.INPUT)):  
            print("Found: Directory")  
        elif os.path.isfile(str(self.INPUT)):  
            print("Found: File")  
        else:  
            self.maybePrint("Warning: Missing Local Path")
        return self.INPUT

    def getPoll(self):
        if isinstance(self.QUESTIONS, str): self.QUESTIONS = self.QUESTIONS.split(",")
        poll = None
        duration = self.DURATION or None
        questions = self.QUESTIONS or None
        poll = {"period":duration,"questions":questions}
        if not duration or not questions: return None
        return poll

    def getSchedule(self):
        if str(self.SCHEDULE) != "None": return self.SCHEDULE
        if  str(self.DATE) != "None":
            if str(self.TIME) != "None":
                self.SCHEDULE = "{}:{}".format(self.DATE,self.TIME)
            else:
                self.SCHEDULE = "{}:{}".format(self.DATE,"00:00")
        return None

    def getTmp(self):
        # mkdir /tmp
        tmp = os.getcwd()
        if self.MOUNT_PATH != None:
            tmp = os.path.join(self.MOUNT_PATH, "tmp")
        else:
            tmp = os.path.join(tmp, "tmp")
        if not os.path.exists(str(tmp)):
            os.mkdir(str(tmp))
        return tmp

    def loginPrompt(self):
        username = input("Twitter Username ({}): ".format(self.USERNAME))
        password = input("Twitter Password: ")
        if username != "" and username != " ":
            self.USERNAME = username
        if password != "" and password != " ":
            self.PASSWORD = password

    def maybePrint(self, text):
        if str(self.VERBOSE) == "True":
            print(text)

    def update_value(self, variable, newValue):
        variable = str(variable).upper().replace(" ","_")
        try:
            # print("Updating: {} = {}".format(variable, newValue))
            setattr(self, variable, newValue)
            # print("Updated: {} = {}".format(variable, getattr(self, variable)))
        except Exception as e:
            maybePrint(e)

SETTINGS = Settings()

def readConf(self, conf):
    posts = False
    self.POSTS = AttrDict()
    with open(conf) as f:
        for line in f:
            if str(line[0]) == "#": continue
            try:
                if "\"" in str(line):
                    spl = re.split("\"*\"", line)
                    key = spl[0] or None
                    val = spl[1] or None
                else:
                    (key, val) = line.split()
                key = key.strip()
                if str(val)[0] == "\"": val = str(val[1:])
                if str(val)[len(val)-1] == "\"": val = str(val[:len(val)-1])
                # print("{} : {} ".format(key.upper(),val))
                if "_post" in key: setattr(self.POSTS, key.upper().replace("_post",""), val)
                else: setattr(self, key.upper(), val)
            except Exception as e:
                pass
                # self.maybePrint(e)
                # print("Warning: Error Parsing Config")

class AttrDict(dict):
    def __init__(self):
        dict.__init__(self)

    # Override getattr and setattr so that they return the values of getitem / setitem
    def __setattr__(self, name, value):
        self[name] = value

    def __getattr__(self, name):
        return self[name]


#!/usr/bin/python3
# 3/25/2019: Skeetzo
# User Class

import json
import time
import os
import threading
from datetime import datetime
from re import sub
from decimal import Decimal

from OnlySnarf import driver as OnlySnarf
from OnlySnarf.settings import SETTINGS as settings

PRICE_MINIMUM = 3

class User:

    def __init__(self, data):
        data = json.loads(json.dumps(data))
        # print(data)
        self.name = data.get('name')
        self.username = data.get('username')
        self.id = data.get('id')
        self.messages_from = data.get('messages_from') or []
        self.messages_to = data.get('messages_to') or []
        self.messages = data.get('messages') or []
        self.preferences = data.get('preferences') or []
        self.last_messaged_on = data.get('last_messaged_on')
        self.sent_images = data.get('sent_images') or []
        self.subscribed_on = data.get('subscribed_on')
        self.isFavorite = data.get('isFavorite') or False
        self.statement_history = data.get('statement_history') or []
        self.started = data.get('started')
        ###### fucking json #####
        self.messages_from = ",".join(self.messages_from).split(",")
        self.messages_to = ",".join(self.messages_from).split(",")
        self.messages = ",".join(self.messages_from).split(",")
        self.preferences = ",".join(self.messages_from).split(",")
        self.sent_images = ",".join(self.messages_from).split(",")
        self.statement_history = ",".join(self.messages_from).split(",")
        #########################
        try:
            settings.maybePrint("User: {} - {} - {}".format(self.name, self.username, self.id))
        except Exception as e:
            settings.maybePrint(e)
            settings.maybePrint("User: {}".format(self.id))

    def sendMessage(self, message="", image=None, price=None):
        try:
            print("Sending Message: {} <- {} - {} - ${}".format(self.id, message, image, price))
            OnlySnarf.message_user(self)
            success = OnlySnarf.message_text(message)
            if not success:
                return
            if image:
                image_name = os.path.basename(image)
                if str(image_name) in self.sent_images:
                    print("Error: Image Already Sent: {} -> {}".format(image, self.id))
                    return False
                success = OnlySnarf.message_image(image)
                if not success: return False
            if price:
                global PRICE_MINIMUM
                if image != None and Decimal(sub(r'[^\d.]', '', price)) < PRICE_MINIMUM:
                    print("Warning: Price Too Low, Free Image")
                    print("Price Minimum: ${}".format(PRICE_MINIMUM))
                else:
                    success = OnlySnarf.message_price(price)
                if not success: return False
            if str(settings.DEBUG) == "True":
                self.sent_images.append("DEBUG")
            else:
                self.sent_images.append(str(image_name))
            if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
                time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
            success = OnlySnarf.message_confirm()
            if not success: return False
            if str(settings.DEBUG) == "False":
                self.last_messaged_on = datetime.now()
            return True
        except Exception as e:
            settings.maybePrint(e)
            return False

    def equals(self, user):
        # print(str(user.username)+" == "+str(self.username))
        if user.username == self.username:
            return True
        return False

    def toJSON(self):
        return json.dumps({
            "name":str(self.name),
            "username":str(self.username),
            "id":str(self.id),
            "messages_from":str(self.messages_from),
            "messages_to":str(self.messages_to),
            "messages":str(self.messages),
            "preferences":str(self.preferences),
            "last_messaged_on":str(self.last_messaged_on),
            "sent_images":str(self.sent_images),
            "subscribed_on":str(self.subscribed_on),
            "isFavorite":str(self.isFavorite)
        })

    # greet user if new
    def greet(self):
        if self.last_messaged_on == None:
            return print("Error: User Not New")
        print("Sending User Greeting: {}".format(self.username))
        self.sendMessage(message=settings.DEFAULT_GREETING)

    # send refresher message to user
    def refresh(self):
        if self.last_messaged_on == None:
            print("Warning: Never Greeted, Greeting Instead")
            return self.greet()
        elif (timedelta(self.last_messaged_on)-timedelta(datetime())).days < 30:
            return print("Error: Refresher Date Too Early - {}".format((timedelta(self.last_messaged_on)-timedelta(datetime())).days))
        print("Sending User Refresher: {}".format(self.username))
        self.sendMessage(message=settings.user_DEFAULT_REFRESHER)

    # saves chat log to user
    def readChat(self):
        print("Reading Chat: {} - {}".format(self.username, self.id))
        messages = OnlySnarf.read_user_messages(self.id)
        self.messages = messages[0]
        # self.messages_and_timestamps = messages[1]
        self.messages_to = messages[2]
        self.messages_from = messages[3]
        settings.maybePrint("Chat Read: {} - {}".format(self.username, self.id))

    # saves statement / payment history
    def statementHistory(self, history):
        print("Reading Statement History: {} - {}".format(self.username, self.id))
        OnlySnarf.read_statements(user=self.id)

    # sets as favorite
    def favor(self):
        print("Favoring: {}".format(self.username))
        self.isFavorite = True

    # unsets as favorite
    def unfavor(self):
        print("Unfavoring: {}".format(self.username))
        self.isFavorite = False

    @staticmethod
    def get_all_users():
        return User.get_active_users()

    # gets users from local or refreshes from onlyfans.com
    @staticmethod
    def get_active_users():
        if str(settings.PREFER_LOCAL) == "True": return read_users_local()
        active_users = []
        users = OnlySnarf.get_users()
        for user in users:
            try:
                user = User(user)
                user = skipUserCheck(user)
                if user is None: continue
                active_users.append(user)
            except Exception as e:
                settings.maybePrint(e)
        settings.maybePrint("pruning memberlist")
        settings.maybePrint("users: {}".format(len(active_users)))
        write_users_local(users=active_users)
        settings.PREFER_LOCAL = True
        return active_users

    @staticmethod
    def get_user_by_username(username):
        if not username or username == None:
            print("Error: Missing Username")
            return None
        users = read_users_local()
        for user in users:
            if str(user.username) == "@u"+str(username) or str(user.username) == "@"+str(username) or str(user.username) == str(username):
                settings.maybePrint("Found User: Local")
                return user
        users = User.get_all_users()
        for user in users:
            if str(user.username) == "@u"+str(username) or str(user.username) == "@"+str(username) or str(user.username) == str(username):
                settings.maybePrint("Found User: Members")
                return user
        return None

    @staticmethod
    def get_favorite_users():
        settings.maybePrint("Getting Fav Users")
        users = User.get_all_users()
        favUsers = []
        favorites = ",".join(str(settings.USERS_FAVORITE))
        for user in users:
            if user in favorites:
                settings.maybePrint("Fav User: {}".format(user.username))
                user = skipUserCheck(user)
                if user is None: continue
                favUsers.append(user)
        return favUsers

    # returns users that have no messages sent to them
    @staticmethod
    def get_new_users():
        settings.maybePrint("Getting New Users")
        users = User.get_all_users()
        newUsers = []
        date_ = datetime.today() - timedelta(days=10)
        for user in users:
            started = datetime.strptime(user.started,"%b %d, %Y")
            # settings.maybePrint("date: "+str(date_)+" - "+str(started))
            if started < date_: continue
            settings.maybePrint("New User: {}".format(user.username))
            user = skipUserCheck(user)
            if user is None: continue
            newUsers.append(user)
        return newUsers

    @staticmethod
    def get_never_messaged_users():
        settings.maybePrint("Getting New Users")
        update_chat_logs()
        users = User.get_all_users()
        newUsers = []
        for user in users:
            if len(user.messages_to) == 0:
                settings.maybePrint("Never Messaged User: {}".format(user.username))
                user = skipUserCheck(user)
                if user is None: continue
                newUsers.append(user)
        return newUsers

    @staticmethod
    def get_recent_users():
        settings.maybePrint("Getting Recent Users")
        users = User.get_all_users()
        i = 0
        users_ = []
        for user in users:
            settings.maybePrint("Recent User: {}".format(user.username))
            user = skipUserCheck(user)
            if user is None: continue
            users_.append(user)
            i += 1
            if i == settings.RECENT_USER_COUNT:
                return users_
        return users_

# gets a list of all subscribed user_ids from local txt
def read_users_local():
    settings.maybePrint("Getting Local Users")
    users = []
    users_ = []
    try:
        with open(settings.USERS_PATH) as json_file:  
            users = json.load(json_file)['users']
        settings.maybePrint("Loaded:")
        for user in users:
            try:
                users_.append(User(json.loads(user)))
            except Exception as e:
                settings.maybePrint(e)
        return users_
    except Exception as e:
        settings.maybePrint(e)
    return users_

def skipUserCheck(user):
    if str(settings.SKIP_USERS) == "None":
        settings.SKIP_USERS = []
    if str(user.id).lower() in settings.SKIP_USERS or str(user.username).lower() in settings.SKIP_USERS:
        settings.maybePrint("skipping: {}".format(user.username))
        return None
    return user

# writes user list to local txt
def write_users_local(users=None):
    if users is None:
        users = User.get_all_users()
    print("Saving Users Locally")
    settings.maybePrint("local data path: "+str(settings.USERS_PATH))
    data = {}
    data['users'] = []
    for user in users:
        settings.maybePrint("Saving: "+str(user.username))
        data['users'].append(user.toJSON())
    try:
        with open(settings.USERS_PATH, 'w') as outfile:  
            json.dump(data, outfile, indent=4, sort_keys=True)
    except FileNotFoundError:
        print("Error: Missing Local Users")
    except OSError:
        print("Error: Missing Local Path")


def delayForThirty():
    settings.maybePrint("30...")
    time.sleep(10)
    settings.maybePrint("20...")
    time.sleep(10)
    settings.maybePrint("10...")
    time.sleep(7)
    settings.maybePrint("3...")
    time.sleep(1)
    settings.maybePrint("2...")
    time.sleep(1)
    settings.maybePrint("1...")
    time.sleep(1)


#!/usr/bin/python3
# 3/28/2019: Skeetzo
import re
import random
import os
import shutil
# import datetime
import json
import sys
import pathlib
import chromedriver_binary
import time
from datetime import datetime, timedelta
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import TimeoutException
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.common.action_chains import ActionChains
from OnlySnarf.user import User
from OnlySnarf.settings import SETTINGS as settings

###################
##### Globals #####
###################

BROWSER = None
ONLYFANS_HOME_URL = 'https://onlyfans.com/'
ONLYFANS_USERS_ACTIVE_URL = "https://onlyfans.com/my/subscribers/active"
SEND_BUTTON_XPATH = "//button[@type='submit' and @class='g-btn m-rounded send_post_button']"
SEND_BUTTON_CLASS = "g-btn.m-rounded.send_post_button"
TWITTER_LOGIN0 = "//a[@class='g-btn m-rounded m-flex m-lg']"
TWITTER_LOGIN1 = "//a[@class='g-btn m-rounded m-flex m-lg btn-twitter']"
TWITTER_LOGIN2 = "//a[@class='btn btn-default btn-block btn-lg btn-twitter']"
USERNAME_XPATH = "//input[@id='username_or_email']"
PASSWORD_XPATH = "//input[@id='password']"
MESSAGE_INPUT_CLASS = ".form-control.b-chat__message-input"
MESSAGE_CONFIRM = "g-btn.m-rounded.b-chat__btn-submit"
MONTHS_INPUT = "form-control.b-fans__trial__select"
DISCOUNT_INPUT = "form-control.b-fans__trial__select"
DISCOUNT_TEXT = "form-control.b-fans__trial__select"
DISCOUNT_USER_BUTTONS = "g-btn.m-rounded.m-border.m-sm"
DISCOUNT_USER_BUTTONS1 = "g-btn.m-rounded"
DISCOUNT_USERS = "g-btn.m-rounded.m-border.m-sm"
DISCOUNT_USERS_ = "b-users__item.m-fans"
EXPIRATION = "g-btn.m-flat.b-make-post__expire-period-btn"
EXPIRATION_PERIODS = "b-make-post__expire__label"
EXPIRATION_SAVE = "g-btn.m-rounded.js-make-post-poll-duration-save"
EXPIRATION_CANCEL = "g-btn.m-rounded.m-border"
ONLYFANS_TWEET = "//label[@for='new_post_tweet_send']"
ONLYFANS_UPLOAD_PHOTO = "fileupload_photo"
ONLYFANS_UPLOAD_MESSAGE_PHOTO = "cm_fileupload_photo"
ONLYFANS_USER_COUNT = "l-sidebar__user-data__item__count"
ONLYFANS_USERS_IDS = "a.g-btn.m-rounded.m-border.m-sm"
ONLYFANS_USERS_STARTEDS = "b-fans__item__list__item"
ONLYFANS_USERS = "g-user-name__wrapper"
ONLYFANS_USERSNAMES = "g-user-username"
ONLYFANS_POST_TEXT_CLASS = "new_post_text_input"
ONLYFANS_PRICE = ".b-chat__btn-set-price"
ONLYFANS_PRICE_INPUT = ".form-control.b-chat__panel__input"
ONLYFANS_PRICE_CLICK = ".g-btn.m-rounded"
ONLYFANS_CHAT_URL = "https://onlyfans.com/my/chats/chat/"
ONLYFANS_UPLOAD_BUTTON = "g-btn.m-rounded.m-border"
ONLYFANS_MESSAGES_FROM = "m-from-me"
ONLYFANS_MESSAGES_ALL = "b-chat__message__text"
ONLYFANS_MESSAGES = "b-chat__message__text"
ONLYFANS_MORE = "g-btn.m-flat.b-make-post__more-btn"
SCHEDULE = "g-btn.m-flat.b-make-post__datepicker-btn"
SCHEDULE_EXISTING_DATE = "vdatetime-calendar__current--month"
SCHEDULE_NEXT_MONTH = "vdatetime-calendar__navigation--next"
SCHEDULE_DAYS = "vdatetime-calendar__month__day"
SCHEDULE_SAVE = "g-btn.m-rounded"
SCHEDULE_HOURS = "vdatetime-time-picker__item.vdatetime-time-picker__item"
SCHEDULE_MINUTES = "vdatetime-time-picker__item"
POLL = "g-btn.m-flat.b-make-post__voting-btn"
POLL_DURATION = "g-btn.m-flat.b-make-post__voting__duration"
POLL_ADD_QUESTION = "g-btn.m-flat.new_vote_add_option"
POLL_SAVE = "g-btn.m-rounded.js-make-post-poll-duration-save"
POLL_CANCEL = "b-dropzone__preview__delete"
POLL_INPUT_XPATH = "//input[@class='form-control']"

#####################
##### Functions #####
#####################

def auth():
    logged_in = False
    global BROWSER
    if not BROWSER or BROWSER == None: logged_in = log_into_OnlyFans()
    else: logged_in = True
    if logged_in == False: print("Error: Failure to Login")
    return logged_in

def goToHome():
    global BROWSER
    if BROWSER == None: return False
    if str(BROWSER.current_url) == str(ONLYFANS_HOME_URL):
        settings.maybePrint("at -> onlyfans.com")
    else:
        settings.maybePrint("goto -> onlyfans.com")
        BROWSER.get(ONLYFANS_HOME_URL)
        WebDriverWait(BROWSER, 60, poll_frequency=6).until(EC.visibility_of_element_located((By.XPATH, SEND_BUTTON_XPATH)))

# Login to OnlyFans
def log_into_OnlyFans():
    print('Logging into OnlyFans')
    username_ = str(settings.USERNAME)
    password_ = str(settings.PASSWORD)
    if not username_ or username_ == "":
        username_ = input("Twitter Username: ")
    if not password_ or password_ == "":
        password_ = input("Twitter Password: ")
        print("Save? y/n")
        save_ = input(">> ")
        if str(save_) != "n": settings.PASSWORD = password__
    if username_ == "" or password_ == "":
        print("Error: Missing Login Info")
        return False
    if str(settings.USERNAME) == "" or settings.USERNAME == None: settings.USERNAME = username_
    if str(settings.PASSWORD) == "" or settings.PASSWORD == None: settings.PASSWORD = password_
    settings.maybePrint("Opening Web Browser")
    options = webdriver.ChromeOptions()
    if str(settings.SHOW_WINDOW) != "True":
        options.add_argument('--headless')
    options.add_argument('--no-sandbox')
    options.add_argument('--disable-dev-shm-usage')
    # options.setExperimentalOption('useAutomationExtension', false);
    options.add_argument('--disable-gpu')
    # BROWSER = webdriver.Chrome(binary=CHROMEDRIVER_PATH, chrome_options=options)
    CHROMEDRIVER_PATH = chromedriver_binary.chromedriver_filename
    os.environ["webdriver.chrome.driver"] = CHROMEDRIVER_PATH
    global BROWSER
    try:
        BROWSER = webdriver.Chrome(chrome_options=options)
    except Exception as e:
        settings.maybePrint(e)
        print("Warning: Missing chromedriver_path, retrying")
        try:
            BROWSER = webdriver.Chrome(CHROMEDRIVER_PATH, chrome_options=options)
        except Exception as e:
            settings.maybePrint(e)
            print("Error: Missing chromedriver_path, exiting")
            return False
    BROWSER.implicitly_wait(10) # seconds
    BROWSER.set_page_load_timeout(1200)    
    def login(opt):
        try:
            BROWSER.get(ONLYFANS_HOME_URL)
            # login via Twitter
            if int(opt)==0:
                twitter = BROWSER.find_element_by_xpath(TWITTER_LOGIN0).click()
            elif int(opt)==1:
                twitter = BROWSER.find_element_by_xpath(TWITTER_LOGIN1).click()
            elif int(opt)==2:
                twitter = BROWSER.find_element_by_xpath(TWITTER_LOGIN2).click()
        except NoSuchElementException as e:
            print("Warning: Login Failure, Retrying")
            login(opt+1)
    try:
        login(0)
        # fill in username
        username = BROWSER.find_element_by_xpath(USERNAME_XPATH).send_keys(username_)
        # fill in password and hit the login button 
        password = BROWSER.find_element_by_xpath(PASSWORD_XPATH)
        password.send_keys(password_)
        password.send_keys(Keys.ENTER)
        WebDriverWait(BROWSER, 60, poll_frequency=6).until(EC.visibility_of_element_located((By.XPATH, SEND_BUTTON_XPATH)))
        print('Login Successful')
        return True
    except Exception as e:
        settings.maybePrint(e)
        print('Login Failure')
        return False

# Reset to home
def reset():
    global BROWSER
    if not BROWSER or BROWSER == None:
        print('OnlyFans Not Open, Skipping Reset')
        return True
    try:
        BROWSER.get(ONLYFANS_HOME_URL)
        print('OnlyFans Reset')
        return True
    except Exception as e:
        settings.maybePrint(e)
        print('Error: Failure Resetting OnlyFans')
        return False

####################
##### Discount #####
####################

# maximum discount = 55%
def discount_user(user, depth=0, discount=10, months=1, tryAll=False):
    if not auth(): return False
    if int(discount) > 55:
        print("Warning: Discount Too High, Max -> 55%")
        discount = 55
    elif int(discount) < 5:
        print("Warning: Discount Too Low, Min -> 5%")
        discount = 5
    if int(months) > 12:
        print("Warning: Months Too High, Max -> 12")
        months = 12
    global BROWSER
    try:
        if str(BROWSER.current_url) == str(ONLYFANS_USERS_ACTIVE_URL):
            settings.maybePrint("at -> /my/subscribers/active")
            BROWSER.execute_script("window.scrollTo(0, 0);")
        else:
            settings.maybePrint("goto -> /my/subscribers/active")
            BROWSER.get(ONLYFANS_USERS_ACTIVE_URL)
        if tryAll: depth = BROWSER.find_element_by_class_name(ONLYFANS_USER_COUNT).get_attribute("innerHTML")
        # settings.maybePrint("Depth: {}".format(depth))
        for n in range(int(int(int(depth)/10)+1)):
            settings.maybePrint("scrolling...")
            BROWSER.execute_script("window.scrollTo(0, document.body.scrollHeight);")
            time.sleep(1)
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failed to Find Users")
        return False
    users = BROWSER.find_elements_by_class_name(DISCOUNT_USERS_)
    print("Discounting User: {} - {}/{}".format(user, depth, len(users)))
    time.sleep(2)
    # get all the users
    user__ = users[0]
    for user_ in users:
        text = user_.get_attribute("innerHTML")
        buttons_ = user_.find_elements_by_class_name(DISCOUNT_USER_BUTTONS)
        if str(user) in text:
            user__ = user_
            break
    ActionChains(BROWSER).move_to_element(user_).perform()
    buttons = user__.find_elements_by_class_name(DISCOUNT_USER_BUTTONS)
    for button in buttons:
        if "Discount" in button.get_attribute("innerHTML"):
            try:
                button.click()
                break
            except Exception as e:
                print("Warning: Unable To Find User, retrying")
                settings.maybePrint(e)
                return discount_user(user, depth=depth, discount=discount, months=months, tryAll=True)
    time.sleep(1)
    buttons_ = BROWSER.find_elements_by_class_name(DISCOUNT_USER_BUTTONS1)
    try:
        (months_, discount_) = BROWSER.find_elements_by_class_name(DISCOUNT_INPUT)
        # removed in 2.10, inputs changed to above
        # months_ = BROWSER.find_element_by_class_name(MONTHS_INPUT)
        # if discount_.get_attribute("value") != "":
            # print("Warning: Existing Discount")
        # discount_.clear()
        for n in range(11):
            discount_.send_keys(str(Keys.UP))
        # if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
        #     time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        for n in range(round(int(discount)/5)-1):
            discount_.send_keys(Keys.DOWN)
        for n in range(11):
            months_.send_keys(str(Keys.UP))
        for n in range(int(months)-1):
            months_.send_keys(Keys.DOWN)
        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        for button in buttons_:
            if "Cancel" in button.get_attribute("innerHTML") and str(settings.DEBUG) == "True":
                button.click()
                print("Skipping: Save Discount (Debug)")
                return True
            elif "Apply" in button.get_attribute("innerHTML") and str(settings.DEBUG) == "False":
                button.click()
                print("Discounted User: {}".format(user))
                return True
    except Exception as e:
        settings.maybePrint(e)
        for button in buttons_:
            if "Cancel" in button.get_attribute("innerHTML"):
                button.click()
                print("Skipping: Save Discount")
                return True

######################
##### Expiration #####
######################

def expiration(period):
    if not auth(): return False
    if int(period) != 1 and int(period) != 3 and int(period) != 7 and int(period) != 30 and int(period) != 99 and str(period) != "No limit":
        print("Error: Missing Expiration")
        return False
    global BROWSER
    try:
        # goToHome()
        if isinstance(period,str) and str(period) == "No limit": period = 99
        print("Expiration:")
        print("- Period: {}".format(period))
        try:
            BROWSER.find_element_by_class_name(ONLYFANS_MORE).click()
        except Exception as e:
            pass
        BROWSER.find_element_by_class_name(EXPIRATION).click()
        nums = BROWSER.find_elements_by_class_name(EXPIRATION_PERIODS)
        for num in nums:
            inner = num.get_attribute("innerHTML")
            if str(inner) == "1 day" and int(period) == 1: num.click()
            if str(inner) == "3 days" and int(period) == 3: num.click()
            if str(inner) == "7 days" and int(period) == 7: num.click()
            if str(inner) == "30 days" and int(period) == 30: num.click()
            if "No limit" in str(inner) and int(period) == 99: num.click()
        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        save = BROWSER.find_element_by_class_name(EXPIRATION_SAVE)
        if str(settings.DEBUG) == "True":
            print("Skipping Expiration (debug)")
            cancels = BROWSER.find_elements_by_class_name(EXPIRATION_CANCEL)
            cancels[1].click() # its the second cancel button
        else:
            save.click()
            print("Expiration Entered")
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failed to enter Expiration")
        return False

####################
##### Messages #####
####################
 
def message_confirm():
    try:
        global BROWSER
        send = WebDriverWait(BROWSER, 60, poll_frequency=10).until(EC.element_to_be_clickable((By.CLASS_NAME, MESSAGE_CONFIRM)))
        if str(settings.DEBUG) == "True":
            if str(settings.DEBUG_DELAY) == "True":
                time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
            print('OnlyFans Message: Skipped (debug)')
            return True
        send.click()
        print('OnlyFans Message: Sent')
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failure to Confirm Message")
        return False

def message_text(text):
    try:
        if not text or text == None or str(text) == "None":
            print("Error: Missing Text")
            return False
        print("Enter text: {}".format(text))
        global BROWSER
        message = BROWSER.find_element_by_css_selector(MESSAGE_INPUT_CLASS)        
        message.send_keys(str(text))
        settings.maybePrint("Text Entered")
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failure to Enter Message")
        return False

def message_image(image):
    try:
        if not image or image == None or str(image) == "None":
            print("Error: Missing Image(s)")
            return False
        print("Enter image(s): {}".format(image))
        global BROWSER
        files = []
        if str(settings.SKIP_DOWNLOAD) == "True":
            print("Warning: Unable to upload, skipped download")
            return True
        if os.path.isfile(str(image)):
            files = [str(image)]
        elif os.path.isdir(str(image)):
            for file in os.listdir(str(image)):
                files.append(os.path.join(os.path.abspath(str(image)),file))
        else:
            print("Error: Missing Image File(s)")
            return False
        if str(settings.SKIP_UPLOAD) == "True":
            print("Skipping Upload")
            return True
        files = files[:int(settings.IMAGE_UPLOAD_MAX_MESSAGES)]
        for file in files:  
            print('Uploading: '+str(file))
            BROWSER.find_element_by_id(ONLYFANS_UPLOAD_MESSAGE_PHOTO).send_keys(str(file))
        try:
            buttons = BROWSER.find_elements_by_class_name(ONLYFANS_UPLOAD_BUTTON)
            if len(buttons) > 0: settings.maybePrint("Warning: Upload Error Message, Closing")
            for butt in buttons:
                if butt.get_attribute("innerHTML").strip() == "Close":
                    butt.click()
                    settings.maybePrint("Success: Upload Error Message Closed")
                    time.sleep(1)
        except Exception as e:
            print("Error: Unable to Upload Images")
            settings.maybePrint(e)
            return False
        settings.maybePrint("Image(s) Entered")
        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failure to Enter Image(s)")
        return False

def message_price(price):
    try:
        if not price or price == None or str(price) == "None":
            print("Error: Missing Price")
            return False
        print("Enter price: {}" .format(price))

        global BROWSER
        WAIT = WebDriverWait(BROWSER, 600, poll_frequency=10).until(EC.element_to_be_clickable((By.CSS_SELECTOR, ONLYFANS_PRICE))).click()
        # BROWSER.find_element_by_css_selector(ONLYFANS_PRICE)
        BROWSER.find_elements_by_css_selector(ONLYFANS_PRICE_INPUT)[1].send_keys(str(price))
        BROWSER.find_elements_by_css_selector(ONLYFANS_PRICE_CLICK)[4].click()
        settings.maybePrint("Price Entered")
        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failure to Enter Price")
        return False

def message_user(user):
    try:
        if not auth(): return False
        userid = user.id
        if not userid or userid == None or str(userid) == "None":
            print("Warning: Missing User ID")
            if not user.username or user.username == None:
                print("Error: Missing User ID & Username")
                return False
            userid = str(user.username).replace("@u","").replace("@","")
            if len(re.findall("[A-Za-z]", userid)) > 0:  
                print("Warning: Invalid User ID")
                if str(settings.DEBUG) == "False":
                    return False
        settings.maybePrint("goto -> /my/chats/chat/%s" % userid)
        global BROWSER
        BROWSER.get(str(ONLYFANS_CHAT_URL)+str(userid))
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failure to Goto User - {}/{}".format(user.id, user.username))
        return False

def read_user_messages(user):
    try:
        if not auth(): return False
        global BROWSER
        # go to onlyfans.com/my/subscribers/active
        message_user(user)
        messages_from_ = BROWSER.find_elements_by_class_name(ONLYFANS_MESSAGES_FROM)
        # print("first message: {}".format(messages_to_[0].get_attribute("innerHTML")))
        # messages_to_.pop(0) # drop self user at top of page
        messages_all_ = BROWSER.find_elements_by_class_name(ONLYFANS_MESSAGES_ALL)
        messages_all = []
        messages_to = []
        messages_from = []
        # timestamps_ = BROWSER.find_elements_by_class_name("b-chat__message__time")
        # timestamps = []
        # for timestamp in timestamps_:
            # settings.maybePrint("timestamp1: {}".format(timestamp))
            # timestamp = timestamp["data-timestamp"]
            # timestamp = timestamp.get_attribute("innerHTML")
            # settings.maybePrint("timestamp: {}".format(timestamp))
            # timestamps.append(timestamp)
        for message in messages_all_:
            settings.maybePrint("all: {}".format(message.get_attribute("innerHTML")))
            messages_all.append(message.get_attribute("innerHTML"))
        messages_and_timestamps = []
        # messages_and_timestamps = [j for i in zip(timestamps,messages_all) for j in i]
        # settings.maybePrint("Chat Log:")
        # for f in messages_and_timestamps:
            # settings.maybePrint(": {}".format(f))
        for message in messages_from_:
            # settings.maybePrint("from1: {}".format(message.get_attribute("innerHTML")))
            message = message.find_element_by_class_name(ONLYFANS_MESSAGES)
            settings.maybePrint("from: {}".format(message.get_attribute("innerHTML")))
            messages_from.append(message.get_attribute("innerHTML"))
        i = 0
        for message in messages_all:
            from_ = False
            to_ = False
            for mess in messages_from:
                if str(message) == str(mess):
                    from_ = True
            for mess in messages_to:
                if str(message) == str(mess):
                    to_ = True
            if not from_:
                # settings.maybePrint("to_: {}".format(message))
                # messages_to[i] = [timestamps[i], message]
                # messages_to[i] = message
                messages_to.append(message)
                # settings.maybePrint("to_: {}".format(messages_to[i]))
            # elif from_:
                # settings.maybePrint("from_: {}".format(message))
                # messages_from[i] = [timestamps[i], message]
                # messages_from[i] = message
                # settings.maybePrint("from_: {}".format(messages_from[i]))
            i += 1
        settings.maybePrint("to: {}".format(messages_to))
        settings.maybePrint("from: {}".format(messages_from))
        settings.maybePrint("Messages From: {}".format(len(messages_from)))
        settings.maybePrint("Messages To: {}".format(len(messages_to)))
        settings.maybePrint("Messages All: {}".format(len(messages_all)))
        return [messages_all, messages_and_timestamps, messages_to, messages_from]
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failure to Read Chat - {}".format(user.username))
        return [[],[],[]]

# update chat logs for all users
def update_chat_logs():
    global USER_CACHE_LOCKED
    USER_CACHE_LOCKED = True
    print("Updating User Chats")
    users = get_users()
    for user in users:
        update_chat_log(user)
    USER_CACHE_LOCKED = False

def update_chat_log(user):
    print("Updating Chat: {} - {}".format(user.username, user.id))
    if not user:
        return print("Error: Missing User")
    user.readChat()

################
##### Poll #####
################

def polling(poll):
    period = poll.get("period")
    questions = poll.get("questions")
    if isinstance(questions, str): questions = questions.split(",\"*\"")
    questions = [n.strip() for n in questions]
    if not auth(): return False
    if int(period) != 1 and int(period) != 3 and int(period) != 7 and int(period) != 30 and int(period) != 99 and str(period) != "No limit":
        print("Error: Missing Duration")
        return False
    if not questions or len(questions) == 0:
        print("Error: Missing Questions")
        return False
    global BROWSER
    try:
        print("Poll:")
        print("- Duration: {}".format(period))
        print("- Questions:\n> {}".format("\n> ".join(questions)))
        try:
            BROWSER.find_element_by_class_name(ONLYFANS_MORE).click()
        except Exception as e:
            pass
        BROWSER.find_element_by_class_name(POLL).click()
        BROWSER.find_element_by_class_name(POLL_DURATION).click()
        nums = BROWSER.find_elements_by_class_name(EXPIRATION_PERIODS)
        for num in nums:
            inner = num.get_attribute("innerHTML")
            if str(inner) == "1 day" and int(period) == 1: num.click()
            if str(inner) == "3 days" and int(period) == 3: num.click()
            if str(inner) == "7 days" and int(period) == 7: num.click()
            if str(inner) == "30 days" and int(period) == 30: num.click()
            if "No limit" in str(inner) and int(period) == 99: num.click()
        save = BROWSER.find_element_by_class_name(POLL_SAVE).click()
        time.sleep(1)
        if len(questions) > 2:
            for question in questions[2:]:
                BROWSER.find_element_by_class_name(POLL_ADD_QUESTION).click()
        questions_ = BROWSER.find_elements_by_xpath(POLL_INPUT_XPATH)
        i = 0
        # print("questions: {}".format(questions))
        for question in list(questions):
            questions_[i].send_keys(str(question))
            time.sleep(1)
            i+=1
        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        if str(settings.DEBUG) == "True":
            print("Skipping Poll (debug)")
            cancel = BROWSER.find_element_by_class_name(POLL_CANCEL)
            cancel.click() # its the second cancel button
        else:
            print("Poll Entered")
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failed to enter Poll")
        return False

# b-dropzone__preview__delete


################
##### Post #####
################

def post(text, expires=None, schedule=False, poll=False):
    try:
        if not auth(): return False
        global BROWSER
        goToHome()
        print("Posting:")
        print("- Text: {}".format(text))
        if expires: expiration(expires)
        if schedule: scheduling(schedule)
        if poll: polling(poll)
        BROWSER.find_element_by_id(ONLYFANS_POST_TEXT_CLASS).send_keys(str(text))
        sends = BROWSER.find_elements_by_class_name(SEND_BUTTON_CLASS)
        for i in range(len(sends)):
            if sends[i].is_enabled():
                sends = sends[i]
        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        if str(settings.DEBUG) == "True":
            print('Skipped: OnlyFans Post (debug)')
            return True
        sends.click()
        # send[1].click() # the 0th one is disabled
        print('OnlyFans Post Complete')
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: OnlyFans Post Failure")
        return False

######################
##### Promotions #####
######################

# or email
def get_new_trial_link():
    if not auth(): return False
    global BROWSER
    # go to onlyfans.com/my/subscribers/active
    try:
        settings.maybePrint("goto -> /my/promotions")
        BROWSER.get(('https://onlyfans.com/my/promotions'))
        trial = BROWSER.find_elements_by_class_name("g-btn.m-rounded.m-sm")[0].click()
        create = BROWSER.find_elements_by_class_name("g-btn.m-rounded")
        for i in range(len(create)):
            if create[i].get_attribute("innerHTML").strip() == "Create":
                create[i].click()
                break

        # copy to clipboard? email to user by email?
        # count number of links
        # div class="b-users__item.m-fans"
        trials = BROWSER.find_elements_by_class_name("b-users__item.m-fans")
        # print("trials")
        # find last one in list of trial link buttons
        # button class="g-btn m-sm m-rounded" Copy trial link
        # trials = BROWSER.find_elements_by_class_name("g-btn.m-sm.m-rounded")
        # print("trials: "+str(len(trials)))
        # trials[len(trials)-1].click()
        # for i in range(len(create)):
        #     print(create[i].get_attribute("innerHTML"))
       
        # find the css for the email / user
        # which there isn't, so, create a 1 person limited 7 day trial and send it to their email
        # add a fucking emailing capacity
        # send it
        link = "https://onlyfans.com/action/trial/$number"
        return link
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failed to Apply Promotion")
        return None

####################
##### Schedule #####
####################

def scheduling(schedule_):
    if not auth(): return False
    global BROWSER
    try:
        if not schedule_:
            print("Error: Missing Schedule")
            return False
        tehdate = datetime.strptime(schedule_, "%m/%d/%Y:%H:%M")
        date = datetime.strptime(schedule_, "%m/%d/%Y:%H:%M").date()
        month_ = tehdate.strftime("%B")
        day_ = tehdate.day
        year_ = tehdate.year
        time_ = datetime.strptime(schedule_, "%m/%d/%Y:%H:%M").time()
        hour_ = datetime.strptime(schedule_, "%m/%d/%Y:%H:%M").hour
        minute_ = datetime.strptime(schedule_, "%m/%d/%Y:%H:%M").minute
        print("Schedule:")
        print("- Date: {}".format(date))
        print("- Time: {}".format(time_))
        try:
            BROWSER.find_element_by_class_name(ONLYFANS_MORE).click()
        except Exception as e:
            pass
        # click schedule
        BROWSER.find_element_by_class_name(SCHEDULE).click()
        searching = True
        while searching:
            existingDate = BROWSER.find_element_by_class_name(SCHEDULE_EXISTING_DATE).get_attribute("innerHTML")
            if str(month_) in str(existingDate) and str(year_) in str(existingDate):
                searching = False
            else:
                BROWSER.find_element_by_class_name(SCHEDULE_NEXT_MONTH).click()
        days = BROWSER.find_elements_by_class_name(SCHEDULE_DAYS)
        for day in days:
            inner = day.get_attribute("innerHTML").replace("<span><span>","").replace("</span></span>","")
            if str(day_) == str(inner):
                day.click()
        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
        saves = BROWSER.find_elements_by_class_name(SCHEDULE_SAVE)
        for save in saves:
            if "Save" in str(save.get_attribute("innerHTML")):
                save.click()
                break
        hours = BROWSER.find_elements_by_class_name(SCHEDULE_HOURS)
        for hour in hours:
            inner = hour.get_attribute("innerHTML")
            if str(hour_) in str(inner) and hour.is_enabled():
                hour.click()
        minutes = BROWSER.find_elements_by_class_name(SCHEDULE_MINUTES)
        for minute in minutes:
            inner = minute.get_attribute("innerHTML")
            if str(minute_) in str(inner) and minute.is_enabled():
                minute.click()
        if str(settings.DEBUG) == "True":
            print("Skipping Schedule (debug)")
            cancel = BROWSER.find_element_by_class_name(EXPIRATION_CANCEL)
            cancel.click() # its the first cancel button
        else:
            saves = BROWSER.find_elements_by_class_name(SCHEDULE_SAVE)
            for save in saves:
                if "Save" in str(save.get_attribute("innerHTML")):
                    save.click()
                    break
            print("Schedule Entered")
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failed to enter Schedule")
        return False

##################
##### Upload #####
##################

# Uploads a directory with a video file or image files to OnlyFans
def upload_to_OnlyFans(path=None, text="", keywords=[], performers=[], expires=False, schedule=False, poll=False):
    try:
        if not auth(): return False
        global BROWSER
        goToHome()
        if not path:
            print("Error: Missing Upload Path")
            return False
        if not text or text == None or str(text) == "None":
            print("Warning: Missing Upload Text")
            text = ""
        text = text.replace(".mp4","")
        text = text.replace(".MP4","")
        text = text.replace(".jpg","")
        text = text.replace(".jpeg","")
        if isinstance(performers, list) and len(performers) > 0: text += " w/ @"+" @".join(performers)
        if isinstance(keywords, list) and len(keywords) > 0: text += " #"+" #".join(keywords)
        text = text.strip()
        print("Uploading:")
        settings.maybePrint("- Path: {}".format(path))
        print("- Keywords: {}".format(keywords))
        print("- Performers: {}".format(performers))
        print("- Text: {}".format(text))
        print("- Tweeting: {}".format(settings.TWEETING))
        if expires: expiration(expires)
        if schedule: scheduling(schedule)
        if poll: polling(poll)
        WAIT = WebDriverWait(BROWSER, 600, poll_frequency=10)
        if str(settings.TWEETING) == "True":
            WAIT.until(EC.element_to_be_clickable((By.XPATH, ONLYFANS_TWEET))).click()
        files = []
        if str(settings.SKIP_DOWNLOAD) == "True":
            print("Warning: Unable to upload, skipped download")
            return True
        if os.path.isfile(str(path)):
            files = [str(path)]
        elif os.path.isdir(str(path)):
            # files = os.listdir(str(path))
            for file in os.listdir(str(path)):
                files.append(os.path.join(os.path.abspath(str(path)),file))
        else:
            print("Error: Unable to parse path")
            return False
        if str(settings.SKIP_UPLOAD) == "True":
            print("Skipping Upload")
            return True
        files = files[:int(settings.IMAGE_UPLOAD_MAX)]
        for file in files:  
            print('Uploading: '+str(file))
            BROWSER.find_element_by_id(ONLYFANS_UPLOAD_PHOTO).send_keys(str(file))
        maxUploadCount = 12 # 2 hours max attempt time
        i = 0
        while True:
            try:                
                WAIT.until(EC.element_to_be_clickable((By.XPATH, SEND_BUTTON_XPATH)))
                break
            except Exception as e:
                # try: 
                #     # check for existence of "thumbnail is fucked up" modal and hit ok button
                #     BROWSER.switchTo().frame("iframe");
                #     BROWSER.find_element_by_class("g-btn m-rounded m-border").send_keys(Keys.ENTER)
                #     print("Error: Thumbnail Missing")
                #     break
                # except Exception as ef:
                #     settings.maybePrint(ef)
                print('uploading...')
                settings.maybePrint(e)
                i+=1
                if i == maxUploadCount and settings.FORCE_UPLOAD is not True:
                    print('Error: Max Upload Time Reached')
                    return False
        try:
            BROWSER.find_element_by_id(ONLYFANS_POST_TEXT_CLASS).send_keys(str(text))
            # first one is disabled
            sends = BROWSER.find_elements_by_class_name(SEND_BUTTON_CLASS)
            for i in range(len(sends)):
                if sends[i].is_enabled():
                    sends = sends[i]
            if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
                time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
            if str(settings.DEBUG) == "True":
                print('Skipped: OnlyFans Upload (debug)')
                return True
            sends.click()
        except:
            settings.maybePrint("Warning: Upload Error Message, Closing")
            try:
                buttons = BROWSER.find_elements_by_class_name(ONLYFANS_UPLOAD_BUTTON)
                for butt in buttons:
                    if butt.get_attribute("innerHTML").strip() == "Close":
                        butt.click()
                        settings.maybePrint("Success: Upload Error Message Closed")
                        send_text = BROWSER.find_element_by_id(ONLYFANS_POST_TEXT_CLASS)
                        send_text.clear()
                        send_text.send_keys(str(text))
                        # first one is disabled
                        sends = BROWSER.find_elements_by_class_name(SEND_BUTTON_CLASS)
                        for i in range(len(sends)):
                            if sends[i].is_enabled():
                                sends = sends[i]
                        if str(settings.DEBUG) == "True" and str(settings.DEBUG_DELAY) == "True":
                            time.sleep(int(settings.DEBUG_DELAY_AMOUNT))
                        if str(settings.DEBUG) == "True":
                            print('Skipped: OnlyFans Upload (debug)')
                            return True
                        sends.click()
            except Exception as e:
                print("Error: Unable to Upload Images")
                settings.maybePrint(e)
                return False
        # send[1].click() # the 0th one is disabled
        print('OnlyFans Upload Complete')
        return True
    except Exception as e:
        settings.maybePrint(e)
        print("Error: OnlyFans Upload Failure")
        return False

#################
##### Users #####
#################

def get_users():
    if not auth(): return False
    global BROWSER
    try:
        if str(BROWSER.current_url) == str(ONLYFANS_USERS_ACTIVE_URL):
            settings.maybePrint("at -> /my/subscribers/active")
        else:
            settings.maybePrint("goto -> /my/subscribers/active")
            BROWSER.get(ONLYFANS_USERS_ACTIVE_URL)
            num = BROWSER.find_element_by_class_name(ONLYFANS_USER_COUNT).get_attribute("innerHTML")
            settings.maybePrint("User count: {}".format(num))
            for n in range(int(int(int(num)/10)+1)):
                settings.maybePrint("scrolling...")
                BROWSER.execute_script("window.scrollTo(0, document.body.scrollHeight);")
                time.sleep(1)
    except Exception as e:
        settings.maybePrint(e)
        print("Error: Failed to Find Users")
        return []
    # avatars = BROWSER.find_elements_by_class_name('b-avatar')
    user_ids = BROWSER.find_elements_by_css_selector(ONLYFANS_USERS_IDS)
    starteds = BROWSER.find_elements_by_class_name(ONLYFANS_USERS_STARTEDS)
    # users = BROWSER.find_elements_by_class_name('g-user-name')
    users = BROWSER.find_elements_by_class_name(ONLYFANS_USERS)
    usernames = BROWSER.find_elements_by_class_name(ONLYFANS_USERSNAMES)
    usernames.pop(0)
    active_users = []
    # settings.maybePrint("user_ids: "+str(len(user_ids)))
    # settings.maybePrint("starteds: "+str(len(starteds)))
    useridsFailed = False
    startedsFailed = False
    if len(user_ids) == 0:
        print("Warning: Unable to find user ids")
        useridsFailed = True
    if len(starteds) == 0:
        print("Warning: Unable to find starting dates")
        startedsFailed = True
    users_ = []
    try:
        user_ids_ = []
        starteds_ = []
        for i in range(len(user_ids)):
            if user_ids[i].get_attribute("href"):
                user_ids_.append(user_ids[i].get_attribute("href"))
        for i in range(len(starteds)):
            text = starteds[i].get_attribute("innerHTML")
            match = re.findall("Started.*([A-Za-z]{3}\s[0-9]{1,2},\s[0-9]{4})", text)
            if len(match) > 0:
                starteds_.append(match[0])
        if len(user_ids_) == 0:
            print("Warning: Unable to find user ids")
            useridsFailed = True
        if len(starteds_) == 0:
            print("Warning: Unable to find starting dates")
            startedsFailed = True
        # settings.maybePrint("ids vs starteds vs avatars: "+str(len(user_ids_))+" - "+str(len(starteds_))+" - "+str(len(avatars)))
        settings.maybePrint("ids vs starteds vs usernames:"+str(len(user_ids_))+" - "+str(len(starteds_))+" - "+str(len(usernames)))
        for i in range(len(users)): # the first is you and doesn't count towards total
            try:
                if not startedsFailed:
                    start = starteds_[i]
                else:
                    start = datetime.now().strftime("%b %d, %Y")
                if not useridsFailed:
                    user_id = user_ids_[i][35:]
                else:
                    user_id = None
                name = users[i]
                username = usernames[i]
                name = str(name.get_attribute("innerHTML")).strip()
                username = str(username.get_attribute("innerHTML")).strip()
                # settings.maybePrint("name: "+str(name))
                # settings.maybePrint("username: "+str(username))
                # settings.maybePrint("user_id: "+str(user_id))
                if str(settings.USERNAME).lower() in str(username).lower():
                    settings.maybePrint("(self): %s = %s" % (settings.USERNAME, username))
                    # first user is always active user but just in case find it in list of users
                    settings.USER_ID = username
                    continue
                users_.append({"name":name, "username":username, "id":user_id, "started":start})
            except Exception as e:
                settings.maybePrint(e)
    except Exception as e:
        settings.maybePrint(e)
    settings.maybePrint("Found: {}".format(len(users_)))
    return users_

################
##### Exit #####
################

def exit(force=False):
    if str(settings.SAVE_USERS) == "True":
        print("Saving and Exiting OnlyFans")
        write_users_local()
    else:
        print("Exiting OnlyFans")
    global BROWSER
    if BROWSER:
        BROWSER.quit()
    BROWSER = None
    print("Browser Closed")
    global logged_in


#!/usr/bin/python3
# 3/28/2019: Skeetzo

import random
import os
import shutil
import datetime
import json
import sys
import subprocess
import pathlib
import io
from subprocess import PIPE, Popen
from pydrive.auth import GoogleAuth
from pydrive.drive import GoogleDrive
from moviepy.editor import VideoFileClip
from apiclient.discovery import build
from httplib2 import Http
from oauth2client import file, client, tools
from apiclient.http import MediaFileUpload,MediaIoBaseDownload
from .settings import SETTINGS as settings

###################
##### Globals #####
###################

AUTH = False
DRIVE = None
PYDRIVE = None
ONE_GIGABYTE = 1000000000
ONE_MEGABYTE = 1000000
FIFTY_MEGABYTES = 50000000
ONE_HUNDRED_KILOBYTES = 100000
OnlyFansFolder_ = None

################
##### Auth #####
################

# Google Auth
def authGoogle():
    settings.maybePrint('Authenticating Google')
    try:
        # PyDrive
        gauth = GoogleAuth()
        # Try to load saved client credentials
        gauth.LoadCredentialsFile(settings.GOOGLE_PATH)
        settings.maybePrint('Loaded: Google Credentials')
        if gauth.credentials is None:
            # Authenticate if they're not there
            gauth.LocalWebserverAuth()
        elif gauth.access_token_expired:
            # Refresh them if expired
            gauth.Refresh()
        else:
            # Initialize the saved creds
            gauth.Authorize()
        # Save the current credentials to a file
        gauth.SaveCredentialsFile(settings.GOOGLE_PATH)
        global PYDRIVE
        PYDRIVE = GoogleDrive(gauth)
        # Drive v3 API
        SCOPES = 'https://www.googleapis.com/auth/drive'
        store = file.Storage(settings.GOOGLE_PATH)
        creds = store.get()
        if not creds or creds.invalid:
            flow = client.flow_from_clientsecrets(settings.PATH_SECRET, SCOPES)
            creds = tools.run_flow(flow, store)
        global DRIVE
        DRIVE = build('drive', 'v3', http=creds.authorize(Http()))
    except Exception as e:
        # settings.maybePrint(e)
        print('Error: Unable to Authenticate w/ Google')
        return False
    settings.maybePrint('Authentication Success') 
    return True

def checkAuth():
    global AUTH
    if not AUTH:
        AUTH = authGoogle()

################################
##### Archiving / Deleting #####
################################

# Deletes online file
def delete_file(file):
    checkAuth()
    if str(settings.SKIP_DOWNLOAD) == "True":
        print("Warning: Unable to Delete, skipped download")
        return True
    if str(settings.FORCE_DELETE) == "True":
        print("Deleting (Forced): {}".format(fileName))
    elif str(settings.DEBUG) == "True":
        print("Skipping Delete (Debug): {}".format(fileName))
        return
    elif str(settings.DELETE_GOOGLE) == "False":
        print('Skipping Delete (Disabled): {}'.format(fileName))
        return
    else:
        print('Deleting: {}'.format(fileName))
    file.Trash()

# Archives posted file / folder
def move_file(file):
    checkAuth()
    if str(settings.SKIP_DOWNLOAD) == "True":
        print("Warning: Unable to Backup, skipped download")
        return True
    if str(settings.FORCE_BACKUP) == "True":
        print("Backing Up (forced): {}".format(file['title']))
    elif str(settings.DEBUG) == "True":
        print("Skipping Backup (debug): {}".format(file['title']))
        return
    elif str(settings.BACKUP) == "False":
        print('Skipping Backup (disabled): {}'.format(file['title']))
        return
    else:
        print('Backing Up: {}'.format(file['title']))
    file['parents'] = [{"kind": "drive#fileLink", "id": str(get_folder_by_name("posted")['id'])}]
    file.Upload()
    print('Google File Backed Up: {}'.format(file['title']))

def move_files(fileName, files):
    checkAuth()
    if str(settings.SKIP_DOWNLOAD) == "True":
        print("Warning: Unable to Backup, skipped download")
        return True
    if str(settings.FORCE_BACKUP) == "True":
        print("Backing Up (forced): {}".format(fileName))
    elif str(settings.DEBUG) == "True":
        print("Skipping Backup (debug): {}".format(fileName))
        return
    elif str(settings.BACKUP) == "False":
        print('Skipping Backup (disabled): {}'.format(fileName))
        return
    else:
        print('Backing Up: {}'.format(fileName))
    title = fileName+" - "+datetime.datetime.now().strftime("%d-%m@%I-%M")
    settings.maybePrint('Moving To: '+title)
    global PYDRIVE
    tmp_folder = PYDRIVE.CreateFile({'title':str(title), 'parents':[{"kind": "drive#fileLink", "id": str(get_folder_by_name("posted")['id'])}],'mimeType':'application/vnd.google-apps.folder'})
    tmp_folder.Upload()
    settings.maybePrint("Backing Up:")
    for file in files:
        settings.maybePrint(" - {}".format(file['title']))
        file['parents'] = [{"kind": "drive#fileLink", "id": tmp_folder['id']}]
        file.Upload()
    print('Google Files Backed Up: {}'.format(title))

###################
##### Folders #####
###################

def create_folders():
    checkAuth()
    print("Creating Folders: {}".format(settings.ROOT_FOLDER))
    OnlyFansFolder = get_folder_root()
    if OnlyFansFolder is None:
        print("Error: Unable To Create Folders")
        return
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(OnlyFansFolder['id'])}).GetList()
    for folder in settings.DRIVE_FOLDERS:
        found = False
        for folder_ in file_list:
            if str(folder) == folder_['title']:
                settings.maybePrint("found: {}".format(folder))
                found = True
        if not found:
            settings.maybePrint("created: {}".format(folder))
            contentFolder = PYDRIVE.CreateFile({"title": str(folder), "parents": [{"id": OnlyFansFolder['id']}], "mimeType": "application/vnd.google-apps.folder"})
            contentFolder.Upload()

def find_folder(parent, folderName):
    checkAuth()
    global PYDRIVE
    if str(parent) != "root":
        parent = parent['id']
    settings.maybePrint("Finding Folder: {}/{}".format(parent, folderName))
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(parent)}).GetList()
    for folder in file_list:
        if str(folder['title']) == str(folderName):
            return folder
    print("Error: Unable to Find Folder - {}".format(folderName))
    return None

def get_files_of_folder(folderName, parent=None):
    folder = get_folder_by_name(folderName, parent=parent)
    global PYDRIVE
    files = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType != 'application/vnd.google-apps.folder'"}).GetList()      
    return files

def get_folder_by_name(folderName, parent=None):
    checkAuth()
    if str(parent) == "galleries" or str(parent) == "images" or str(parent) == "videos" or str(parent) == "performers":
        parent = get_folder_by_name(parent)
    settings.maybePrint("Getting Folder: {}".format(folderName))
    global PYDRIVE
    if parent is None:
        parent = get_folder_root()
    file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(parent['id'])}).GetList()
    for folder in file_list:
        if str(folder['title'])==str(folderName):
            settings.maybePrint("Found Folder: {}".format(folderName))
            return folder
    if str(settings.CREATE_DRIVE) == "False":
        settings.maybePrint("Skipping: Create Missing Folder - {}".format(folderName))
        return None
    # create if missing
    folder = PYDRIVE.CreateFile({"title": str(folderName), "mimeType": "application/vnd.google-apps.folder", "parents": [{"kind": "drive#fileLink", "id": OnlyFansFolder}]})
    folder.Upload()
    settings.maybePrint("Created Folder: {}".format(folderName))
    return folder

def get_folders_of_folder(folderName, parent=None):
    checkAuth()
    settings.maybePrint("Getting Folders of: {}".format(folderName))
    global PYDRIVE
    folders = []
    folder_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(get_folder_by_name(folderName, parent=parent)['id'])}).GetList()
    for folder in folder_list:
        file_list = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false".format(folder['id'])}).GetList()
        if len(file_list) > 0:
            settings.maybePrint("Found Folder: {}".format(folder['title']))
            folders.append(folder)
        else:
            settings.maybePrint("Found Folder (empty): {}".format(folder['title']))
    return folders

# Creates the OnlyFans folder structure
def get_folder_root():
    checkAuth()
    global PYDRIVE
    global OnlyFansFolder_
    if OnlyFansFolder_ is not None:
        return OnlyFansFolder_
    OnlyFansFolder = None
    if settings.DRIVE_PATH is not None:
        mount_root = "root"
        root_folders = settings.DRIVE_PATH.split("/")
        settings.maybePrint("Mount Folders: {}".format("/".join(root_folders)))    
        for folder in root_folders:
            mount_root = find_folder(mount_root, folder)
            if mount_root is None:
                mount_root = "root"
                print("Warning: Drive Mount Folder Not Found")
                break
        mount_root = find_folder(mount_root, settings.ROOT_FOLDER)
        if mount_root is None:
            mount_root = {"id":"root"}
            print("Warning: Drive Mount Folder Not Found")
        else:
            settings.maybePrint("Found Root (alt): {}/{}".format(settings.DRIVE_PATH, settings.ROOT_FOLDER))
        OnlyFansFolder = mount_root
    else:
        file_list = PYDRIVE.ListFile({'q': "'root' in parents and trashed=false"}).GetList()
        for folder in file_list:
            if str(folder['title']) == str(settings.ROOT_FOLDER):
                OnlyFansFolder = folder
                settings.maybePrint("Found Root: {}".format(settings.ROOT_FOLDER))
    if OnlyFansFolder is None:
        print("Creating Root: {}".format(settings.ROOT_FOLDER))
        OnlyFansFolder = PYDRIVE.CreateFile({"title": str(settings.ROOT_FOLDER), "mimeType": "application/vnd.google-apps.folder"})
        OnlyFansFolder.Upload()
    OnlyFansFolder_ = OnlyFansFolder
    return OnlyFansFolder_

####################
##### Download #####
####################

# Download File
def download_file(file, REPAIR=False):
    if not file:
        print("Error: Missing File")
        return
    tmp = settings.getTmp()
    if str(settings.SKIP_DOWNLOAD) == "True":
        print("Skipping Download (debug)")
        settings.update_value("input",tmp)
        return {"path":tmp}
    checkAuth()
    print('Downloading File: {}'.format(file['title']))
    # download file
    name = os.path.splitext(file['title'])[0]
    ext = os.path.splitext(file['title'])[1].lower()
    if not ext:
        ext = '.mp4'
        settings.maybePrint('ext (default): '+str(ext))
    else:
        settings.maybePrint('ext: '+str(ext))
    name = "{}{}".format(name, ext)
    tmp = os.path.join(tmp, name)
    settings.maybePrint("name: {}".format(name))
    settings.maybePrint('path: '+str(tmp))
    if str(ext).lower() == ".mp4":
        try:
            with open(tmp, 'w+b') as output:
                # print("8",end="",flush=True)
                file_id = file['id']
                request = DRIVE.files().get_media(fileId=file['id'])
                downloader = MediaIoBaseDownload(output, request)
                # print("=",end="",flush=True)
                done = False
                while done is False:
                    # print("=",end="",flush=True)
                    status, done = downloader.next_chunk()
                    if str(settings.VERBOSE) == "True":
                        print("Downloading: %d%%\r" % (status.progress() * 100),end="")
                # print("D")
                print("Download Complete: Regular")
        except Exception as e:
            settings.maybePrint(e)
            file.GetContentFile(tmp)
            print("Download Complete: Alternative")
        if REPAIR:
            tmp = repair(tmp)
        global FIFTY_MEGABYTES
        if int(os.stat(str(tmp)).st_size) >= FIFTY_MEGABYTES or settings.FORCE_REDUCTION: # greater than 1GB
            tmp = reduce(tmp)
        tmp = thumbnail_fix(tmp)
    else:
        file.GetContentFile(tmp)
        print("Download Complete: Alt")
    ### Finish ###
    if not os.path.isfile(str(tmp)):
        print("Error: Missing Downloaded File")
        return False
    size = os.path.getsize(tmp)
    settings.maybePrint("File Size: {}kb - {}mb".format(size/1000, size/1000000))
    global ONE_MEGABYTE
    if size <= ONE_MEGABYTE:
        settings.maybePrint("Warning: Small File Size")
    global ONE_HUNDRED_KILOBYTES
    if size <= ONE_HUNDRED_KILOBYTES:
        settings.maybePrint("Error: File Size Too Small")
        print("Error: Download Failure")
        return False
    settings.update_value("input",tmp)
    print('Downloaded: File')
    return {"path":tmp}

# Download Gallery
def download_gallery(folder):
    if not folder:
        print("Error: Missing Folder")
        return
    tmp = settings.getTmp()
    if str(settings.SKIP_DOWNLOAD) == "True":
        print("Skipping Download (debug)")
        settings.update_value("input",tmp)
        return {"path":tmp,"files":[]}
    checkAuth()
    print('Downloading Gallery: {}'.format(folder['title']))
    # download folder
    global PYDRIVE
    file_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()
    folder_size = len(file_list)
    settings.maybePrint('Folder size: '+str(folder_size))
    settings.maybePrint('Upload limit: '+str(settings.IMAGE_UPLOAD_LIMIT))
    if int(folder_size) == 0:
        print('Error: Empty Folder')
        return False
    random.shuffle(file_list)
    file_list = file_list[:int(settings.IMAGE_UPLOAD_LIMIT)]
    i = 1
    for file in sorted(file_list, key = lambda x: x['title']):
        print('Downloading: {} ({}/{})'.format(file['title'], i, folder_size))
        settings.maybePrint('filePath: '+os.path.join(tmp, str(file['title'])))
        file.GetContentFile(os.path.join(tmp, str(file['title'])))
        i+=1
    print('Downloaded: Gallery')
    settings.update_value("input",tmp)
    return {"path":tmp,"files":file_list}

def download_message_image(folderName="random"):
    print('Fetching Image')
    file = {}
    data = {}
    try:
        file = get_message_image(folderName)
        data = download_file(file.get("file"))
    except Exception as e:
        settings.maybePrint(e)
        return {}
    settings.update_value("input",data.get("path"))
    return {"path":data.get("path"), "file":file.get("file")}

def download_content(folder):
    if not folder:
        print("Error: Missing Folder")
        return
    checkAuth()
    print('Downloading Content: {}'.format(folder['title']))
    # mkdir /tmp
    global PYDRIVE
    content_title = folder['title']
    # download folder
    # file_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png' or mimeType contains 'video/mp4')"}).GetList()
    image_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()
    video_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'video/mp4'"}).GetList()
    file_list = []
    for i in image_list: file_list.append(i)
    for v in video_list: file_list.append(v)
    folder_size = len(file_list)
    settings.maybePrint('Images: {}'.format(len(image_list)))
    settings.maybePrint('Videos: {}'.format(len(video_list)))
    settings.maybePrint('Total: {}'.format(folder_size))
    # settings.maybePrint("Files: {}".format(file_list))
    content = {}
    # if it contains only 1 file, I want to download a file
    if int(folder_size) == 1:
        file = file_list[0]
        content = download_file(file)
    # if it contains multiple images, I want to download a gallery
    elif int(len(image_list)) > 1:
        file = content_found
        content = download_gallery(file)
    # if it contains only 1 image, I want to download a file
    elif int(len(image_list)) == 1:
        file = image_list[0]
        content = download_gallery(file)
    # if it contains at least 1 video, I want to download a random file
    elif int(len(video_list)) >= 1:
        file = file_list[0]
        content = download_file(file)
    elif int(folder_size) == 0:
        print("Warning: Missing Files")
    content["file"] = file
    content["keywords"] = content_title
    print('Downloaded: Content')
    return content

# Download Performer
def download_performer(folder):
    if not folder:
        print("Error: Missing Folder")
        return
    checkAuth()
    print('Downloading Performer: {}'.format(folder['title']))
    # mkdir /tmp
    tmp = settings.getTmp()
    global PYDRIVE
    content_folders = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    content_found = []
    random_content = None
    content_title = None
    for folder in content_folders:
        settings.maybePrint('content: '+folder['title'])
        content_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png' or mimeType contains 'video/mp4')"}).GetList()
        # video_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()
        # image_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'video/mp4'"}).GetList()
        if len(content_list) == 0:
            settings.maybePrint('- skipping empty content: '+folder['title'])
        if len(content_list) > 0:
            settings.maybePrint('- content galleries found: '+folder['title'])
            content_found.append(folder)
    if len(content_found)==0:
        print('Warning: Missing Content Folder')
        return {}
    content_found = random.choice(content_found)
    content_title = content_found['title']
    settings.maybePrint("Folder: {}".format(content_title))
    content = download_content(content_found)
    print('Downloaded: Performer')
    return content

# Download Scene
def download_scene(sceneFolder):
    checkAuth()
    print('Downloading Scene')
    tmp = settings.getTmp()
    global PYDRIVE
    content = None
    preview = None
    folder_list = PYDRIVE.ListFile({'q': "'"+sceneFolder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
    for folder in folder_list:
        if folder['title'] == "content":
            content = folder
            settings.maybePrint("Content Folder: "+folder['id'])
        elif folder['title'] == "preview":
            preview = folder
            settings.maybePrint("Preview Folder: "+folder['id'])
    data = PYDRIVE.ListFile({'q': "'"+sceneFolder['id']+"' in parents and trashed=false and mimeType contains 'text/plain'"}).GetList()
    if len(data) == 0:
        print("Error: Missing Scene Data")
        return
    data = data[0]
    if data is None:
        print("Error: Missing Scene Data")
        return
    if content is None:
        print("Error: Missing Scene Content Folder")
        return
    if preview is None:
        print("Error: Missing Scene Preview Folder")
        return
    # tries to download as gallery first, then finds first video file
    content_ = None
    try:
        content_ = download_gallery(content)
    except Exception as e:
        print("1:" +str(e))
    if content_ is None:
        try:
            content_ = PYDRIVE.ListFile({'q': "'"+content['id']+"' in parents and trashed=false and mimeType != 'application/vnd.google-apps.folder'"}).GetList()
            content_ = download_file(content_[0])
        except Exception as e:
            print("2: "+str(e))
    content = content_
    if content is None:
        print("Error: Unable to Find Content")
        return
    # move content to tmp/content
    tmp_content = os.path.join(content[1], "content")
    settings.maybePrint("Old Content Path: {}".format(content[1]))
    settings.maybePrint("New Content Path: {}".format(tmp_content))
    os.mkdir(tmp_content)
    for file in os.listdir(content[1]):
        file = os.path.join(content[1], file) 
        settings.maybePrint("Moving: {}".format(file))
        shutil.move(file, tmp_content)
    data.GetContentFile(os.path.join(tmp, "data.json"))
    # read data.json    
    with open(os.path.join(tmp, "data.json"), 'r', encoding='utf-8') as f:
        data = json.load(f)
    settings.maybePrint("data.json: {}".format(data))
    preview = PYDRIVE.ListFile({'q': "'"+preview['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()
    if len(preview) == 0:
        print("Error: Missing Scene Preview")
        return
    preview = preview[0]
    preview = download_file(preview)
    if "mp4" in preview:
        preview = thumbnail_fix(preview)
    if data is None:
        print("Error: Missing Scene Data")
        return
    if content is None:
        print("Error: Missing Scene Content")
        return
    if preview is None:
        print("Error: Missing Scene Preview")
        return
    print('Downloaded: Scene')
    return {"path":tmp_content,"preview":preview,"data":data,"content":content}

###############
##### Get #####
###############

# gets all the images in the images folders
def get_images():
    checkAuth()
    print('Getting Images')
    global PYDRIVE
    images_folder = get_folder_by_name("images")
    random_folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(images_folder['id'])}).GetList()
    images_list = []
    for folder in random_folders:
        images_list.append([images_folder, folder])
        settings.maybePrint('checking folder: '+folder['title'])
        images_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()      
        for image_file in images_list_tmp:
            # images_list.append({"folder":folder['title'],"folder_id":folder['id'],"id":image_file['id'],"image":image_file['title']})
            images_list.append([folder, image_file])
    if len(images_list)==0:
        print('Warning: Missing Message Files')
        return
    return images_list

# gets all the images in the messages folders
def get_message_image(folderName):
    checkAuth()
    print('Getting Message Image: {}'.format(folderName))
    global PYDRIVE
    random_folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(get_folder_by_name("messages")['id'])}).GetList()
    images_list = []
    random_image = None
    folder_name = None
    for folder in random_folders:
        if folder['title'] != str(folderName):
            continue
        if str(settings.VERBOSE) == "True":
            print('checking folder: '+folder['title'],end="")
        images_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()      
        if settings.BYKEYWORD != None and str(settings.BYKEYWORD) != str(folder['title']):
            settings.maybePrint('-> not keyword')
            continue
        if len(images_list_tmp)>0:
            images_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    if len(images_list)==0:
        print('Error: Missing Image File')
        return {}
    random_image = random.choice(images_list)
    folder_name = random_image['title'];
    print('Messages Folder: '+random_image['title'])
    random_image = PYDRIVE.ListFile({'q': "'"+random_image['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()
    random_image = random.choice(random_image)
    print('Messages Image: '+random_image['title'])
    return {"file":random_image}

# Downloads random image from Google Drive
def get_random_image():
    checkAuth()
    print('Getting Random Image')
    global PYDRIVE
    random_folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(get_folder_by_name("images")['id'])}).GetList()
    images_list = []
    random_image = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking folder: '+folder['title'],end="")
        images_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()      
        if settings.BYKEYWORD != None and str(settings.BYKEYWORD) != str(folder['title']):
            settings.maybePrint('-> not keyword')
            continue
        if len(images_list_tmp)>0:
            images_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    if len(images_list)==0:
        print('Error: Missing Image File')
        return {}
    random_image = random.choice(images_list)
    folder_name = random_image['title'];
    print('Random Folder: '+random_image['title'])
    random_image = PYDRIVE.ListFile({'q': "'"+random_image['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()
    random_image = random.choice(random_image)
    print('Random Image: '+random_image['title'])
    return {"file":random_image,"keywords":folder_name}

# Downloads random gallery from Google Drive
def get_random_gallery():
    checkAuth()
    print('Getting Random Gallery')
    global PYDRIVE
    random_folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(get_folder_by_name("galleries")['id'])}).GetList()
    folder_list = []
    random_gallery = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking galleries: {}'.format(folder['title']),end="")
        gallery_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        if settings.BYKEYWORD != None and str(settings.BYKEYWORD) != str(folder['title']):
            settings.maybePrint('-> not keyword')
            continue
        if len(gallery_list_tmp)>0:
            folder_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    random.shuffle(folder_list)
    for folder in folder_list:
        if str(settings.VERBOSE) == "True":
            print('checking gallery: {}'.format(folder['title']),end="")
        gallery_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        random_gallery_tmp = random.choice(gallery_list_tmp)
        gallery_list_tmp_tmp = PYDRIVE.ListFile({'q': "'"+random_gallery_tmp['id']+"' in parents and trashed=false and (mimeType contains 'image/jpeg' or mimeType contains 'image/jpg' or mimeType contains 'image/png')"}).GetList()
        if len(gallery_list_tmp_tmp)>0:
            folder_name = folder['title']
            random_gallery = random_gallery_tmp
            settings.maybePrint(" -> found")
        else:
            settings.maybePrint(" -> empty")
    if not random_gallery:
        print('Error: Missing Gallery Folder')
        return {}
    print('Random Gallery: '+random_gallery['title'])
    return {"file":random_gallery,"keywords":folder_name}

# Downloads random performer from Google Drive
def get_random_performer():
    checkAuth()
    print('Getting Random Performer')
    global PYDRIVE
    random_folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(get_folder_by_name("performers")['id'])}).GetList()
    performer_list = []
    random_performer = None
    # print('random folders: '+str(random_folders))
    for folder in random_folders:
        # random_folder_folder = random.choice(random_folders)
        settings.maybePrint('random performer: '+folder['title'])
        performer_content_list = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        # print('random folders: '+str(performer_list))
        if settings.BYKEYWORD != None and str(settings.BYKEYWORD) != str(folder['title']): 
            settings.maybePrint('- skipping nonkeyword: '+folder['title'])
            continue
        if len(performer_content_list)==0:
            settings.maybePrint('- skipping empty performer: '+folder['title'])
        elif len(performer_content_list)>0:
            settings.maybePrint('- performer found: '+folder['title'])
            performer_list.append(folder)
    if len(performer_list)==0:
        print('Error: Missing Performer Folder')
        return {}
    random_performer = random.choice(performer_list)
    print('Random Performer: '+random_performer['title'])
    return {"file":random_performer}

# Downloads random video from Google Drive
def get_random_video():
    checkAuth()
    print('Getting Random Video')
    global PYDRIVE
    random_folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(get_folder_by_name("videos")['id'])}).GetList()
    video_list = []
    random_video = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking folder: '+folder['title'],end="")
        video_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'video/mp4'"}).GetList()
        if settings.BYKEYWORD != None and str(settings.BYKEYWORD) != str(folder['title']):
            settings.maybePrint('-> not keyword')
            continue
        if len(video_list_tmp)>0:
            video_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    if len(video_list)==0:
        print('Error: Missing Video File')
        return {}
    random_video = random.choice(video_list)
    folder_name = random_video['title'];
    print('Random Folder: '+random_video['title'])
    random_video = PYDRIVE.ListFile({'q': "'"+random_video['id']+"' in parents and trashed=false and mimeType contains 'video/mp4'"}).GetList()
    random_video = random.choice(random_video)
    print('Random Video: '+random_video['title'])
    return {"file":random_video,"keywords":folder_name}

# Downloads random scene from Google Drive
def get_random_scene():
    checkAuth()
    print('Getting Random Scene')
    global PYDRIVE
    random_folders = PYDRIVE.ListFile({'q': "'{}' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'".format(get_folder_by_name("scenes")['id'])}).GetList()
    folder_list = []
    random_scene = None
    folder_name = None
    for folder in random_folders:
        if str(settings.VERBOSE) == "True":
            print('checking scenes: '+folder['title'],end="")
        scene_list_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'application/vnd.google-apps.folder'"}).GetList()
        if settings.BYKEYWORD != None and str(settings.BYKEYWORD) != str(folder['title']):
            settings.maybePrint('-> not keyword')
            continue
        if len(scene_list_tmp)>0:
            folder_list.append(folder)
            settings.maybePrint(" -> added")
        else:
            settings.maybePrint(" -> empty")
    random.shuffle(folder_list)
    for folder in folder_list:
        if str(settings.VERBOSE) == "True":
            print('checking scene: '+folder['title'],end="")
        scene_list_tmp_tmp = PYDRIVE.ListFile({'q': "'"+folder['id']+"' in parents and trashed=false and mimeType contains 'text/plain'"}).GetList()
        if len(scene_list_tmp_tmp)>0:
            folder_name = folder['title']
            random_scene = folder
            settings.maybePrint(" -> found")
        else:
            settings.maybePrint(" -> empty")
    if not random_scene:
        print('Error: Missing Scene Folders')
        return {}
    print('Random Scene: '+random_scene['title'])
    return {"file":random_scene,"keywords":folder_name}

##################
##### Random #####
##################

def random_download(fileChoice):
    print('Random: {}'.format(fileChoice))
    file = None
    file_ = None
    data = None
    keywords = None
    performers = None
    try:
        if fileChoice == 'image':
            file = get_random_image()
            data = download_file(file.get("file"))
        elif fileChoice == 'gallery':
            file = get_random_gallery()
            data = download_gallery(file.get("file"))
        elif fileChoice == 'performer':
            file = get_random_performer()
            data = download_performer(file.get("file"))
            performers = file.get("file").get("title").split(" ")
            file_ = data.get("file")
            keywords = data.get("keywords")
        elif fileChoice == 'scene':
            file = get_random_scene()
            return download_scene(file.get("file"))
        elif fileChoice == 'video':
            file = get_random_video()
            data = download_file(file.get("file"))
        else:
            return print("Error: Missing File Choice")
        if file == None or data == None:
            print("Error: Missing Random File(s)")
            return {}
    except Exception as e:
        settings.maybePrint(e)
        return {}
    return {"path":data.get("path"), "file":file_ or file.get("file"), "files":data.get("files"), "keywords":keywords or file.get("keywords"), "performers":performers}

##################
##### Upload #####
##################

def upload_file(path=None, parent=None):
    checkAuth()
    file = os.path.basename(path)
    filename = os.path.splitext(file)[0]
    ext = os.path.splitext(file)[1].lower()
    mimetype = None
    if str(settings.FORCE_BACKUP) == "True":
        print("Google Upload (forced): {}".format(filename))
    elif str(settings.DEBUG) == "True":
        print("Skipping Google Upload  (debug): {}".format(filename))
        return
    elif str(settings.BACKUP) == "False":
        print('Skipping Google Upload  (disabled): {}'.format(filename))
        return
    else:
        print('Google Upload: {}'.format(filename))
    if "mp4" in ext:
        mimetype = "video/mp4"
    elif "jpg" in ext or "jpeg" in ext:
        mimetype = "image/jpeg"
    if not mimetype:
        print("Error: Missing Mimetype")
        return
    if not parent: parent = get_folder_by_name("posted")
    if not parent:
        print("Error: Missing Posted Folder")
        return
    # file_metadata = {
    #     'name': str(filename),
    #     'mimeType': str(mimetype),
    #     'parents': [{"kind": "drive#fileLink", "id": str(parent['id'])}]
    # }
    # media = MediaFileUpload(path, mimetype=str(mimetype), resumable=True)
    # file = DRIVE.files().create(body=file_metadata, media_body=media, fields='id').execute()
    file = PYDRIVE.CreateFile({'title':str(filename), 'parents':[{"kind": "drive#fileLink", "id": str(parent['id'])}],'mimeType':str(mimetype)})
    file.Upload()
    # print('File ID: {}'.format(file.get('id')))

def upload_gallery(path=None):
    parent = get_folder_by_name("posted")
    if not parent:
        print("Error: Missing Posted Folder")
        return
    if str(settings.FORCE_BACKUP) == "True":
        print("Google Upload (forced): {}".format(path))
    elif str(settings.DEBUG) == "True":
        print("Skipping Google Upload  (debug): {}".format(path))
        return
    elif str(settings.BACKUP) == "False":
        print('Skipping Google Upload  (disabled): {}'.format(path))
        return
    else:
        print('Google Upload: {}'.format(path))
    file_metadata = {
        'name': str(datetime.datetime.now()),
        'mimeType': str("application/vnd.google-apps.folder"),
        'parents': [{"kind": "drive#fileLink", "id": str(parent['id'])}]
    }
    tmp_folder = PYDRIVE.CreateFile({'title':str(datetime.datetime.now()), 'parents':[{"kind": "drive#fileLink", "id": str(parent['id'])}],'mimeType':'application/vnd.google-apps.folder'})
    tmp_folder.Upload()
    # media = MediaFileUpload(path, mimetype="application/vnd.google-apps.folder", resumable=True)
    # parent = DRIVE.files().create(body=file_metadata, fields='id').execute()
    files = [os.path.join(path, f) for f in os.listdir(path) if os.path.isfile(os.path.join(path, f))]
    for file in files:
        upload_file(path=file, parent=tmp_folder)

def upload_input(path=None):
    if not path: path = settings.INPUT
    if os.path.isdir(path):
        upload_gallery(path=path)
    else:
        upload_file(path=path)

##################
##### FFMPEG #####
##################

def repair(path):
    if str(settings.SKIP_REPAIR) == "True":
        print("Warning: Skipping Repair")
        return path
    repairedPath = str(path).replace(".mp4", "_fixed.mp4")
    try:
        print("Repairing: {} <-> {}".format(path, settings.WORKING_VIDEO))
        if str(settings.VERBOSE) == "True":
            subprocess.call(['untrunc', str(settings.WORKING_VIDEO), str(path)]).communicate()
        else:
            subprocess.Popen(['untrunc', str(settings.WORKING_VIDEO), str(path)],stdin=FNULL,stdout=FNULL)
    except AttributeError:
        if os.path.isfile(str(path)+"_fixed.mp4"):
            shutil.move(str(path)+"_fixed.mp4", repairedPath)
            print("Repair Complete")
    except:
        settings.maybePrint(sys.exc_info()[0])
        print("Warning: Skipping Repair")
        return path
    print("Repair Successful")
    return str(repairedPath)

def reduce(path):
    if str(settings.SKIP_REDUCE) == "True":
        print("Warning: Skipping Reduction")
        return path
    reducedPath = str(path).replace(".mp4", "_reduced.mp4")
    try:
        settings.maybePrint("Reducing: {}".format(path))
        try:
            clip = VideoFileClip(str(path))
            settings.maybePrint("Length: {}".format(clip.duration))
            bitrate = 1000000000 / int(clip.duration)
            settings.maybePrint("Bitrate: {}".format(bitrate))
        except FileNotFoundError:
            print("Error: Missing File to Reduce")
            return path
        loglevel = "quiet"
        if str(settings.VERBOSE) == "True":
            loglevel = "debug"
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-err_detect', 'ignore_err', '-y', '-i', str(path), '-c', 'copy', '-c:v', 'libx264', '-c:a', 'aac', '-strict', '2', '-crf', '26', '-b:v', str(bitrate), str(reducedPath)])
        # p.communicate()
    except FileNotFoundError:
        print("Warning: Ignoring Fixed Video")
        return reduce(str(path).replace(".mp4", "_fixed.mp4"))
    except Exception as e:
        settings.maybePrint(e)
        if "Conversion failed!" in str(e):
            print("Error: Conversion Failure")
            return path                    
    print("Reduction Complete")
    originalSize = os.path.getsize(str(path))
    newSize = os.path.getsize(str(reducedPath))
    print("Original Size: {}kb - {}mb".format(originalSize/1000, originalSize/1000000))
    print("Reduced Size: {}kb - {}mb".format(newSize/1000, newSize/1000000))
    if int(originalSize) < int(newSize):
        print("Warning: Original Size Smaller")
        return path
    if int(newSize) == 0:
        print("Error: Missing Reduced File")
        return path
    return reducedPath

def thumbnail_fix(path):
    if str(settings.THUMBNAILING_PREVIEW) == "False":
        print("Warning: Preview Thumbnailing Disabled")
        return path
    try:
        print("Thumbnailing: {}".format(path))
        loglevel = "quiet"
        if str(settings.VERBOSE) == "True":
            loglevel = "debug"
        thumbnail_path = os.path.join(os.path.dirname(str(path)), 'thumbnail.png')
        settings.maybePrint("thumbnail path: {}".format(thumbnail_path))
        p = subprocess.call(['ffmpeg', '-loglevel', str(loglevel), '-i', str(path),'-ss', '00:00:00.000', '-vframes', '1', str(thumbnail_path)])
        p.communicate()
        print("Thumbnailing Complete")
        return thumbedPath
    except FileNotFoundError:
        print("Warning: Ignoring Thumbnail")
    except AttributeError:
        print("Thumbnailing: Captured PNG")
    except:
        settings.maybePrint(sys.exc_info()[0])
        print("Error: Thumbnailing Fuckup")    